// -*- C++ -*-
/***************************************************************************
                          chathand.cpp  -  description
                             -------------------
    begin                : Fre Jun 22 17:19:03 CEST 2001
    copyright            : (C) 2001 by Daniel Hermann
    email                : Daniel.Hermann@stud.uni-karlsruhe.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

//
// System
//

#include <stdio.h>
#include <string.h>
#include <ctype.h>

// 
// Common+Handler
//

#include "packets.h"
#include "shared.h"

#include "stdinhand.h"
#include "chathand.h"

//
// Game
//

#include "game.h"
#include "player.h"


extern Game *game;

/**************************************************************************
...
**************************************************************************/
void handle_chat_msg(Player *pplayer, 
		     struct packet_generic_message *packet)
{
  int i;
  struct packet_generic_message genmsg;
  char *cp;

  /* this loop to prevent players from sending multiple lines
   * which can be abused */
  genmsg.x = -1;
  genmsg.y = -1;
  genmsg.event =-1;
  for(cp=packet->message; *cp; ++cp)
    if(!isprint(*cp & 0x7f)) {
      *cp='\0';
      break;
    }

  /* Server commands are prefixed with '/', which is an obvious
     but confusing choice: even before this feature existed,
     novice players were trying /who, /nick etc.
     So consider this an incentive for IRC support,
     or change it in stdinhand.h - rp
  */
  if (packet->message[0] == SERVER_COMMAND_PREFIX) {
    /* pass it to the command parser, which will chop the prefix off */
    handle_stdin_input(pplayer, packet->message);
    return;
  }

  /* Want to allow private messages with "player_name: message",
     including unambiguously abbreviated player name, but also want
     to allow sensible use of ':' within messages, and _also_ want to
     notice intended private messages with (eg) mis-spelt player name.

     Approach:
     
     If there is no ':', or ':' is first on line,
          message is global (send to all players)
     else if the part before ':' matches one player unambiguously,
          send to that one player
     else if it matches several players ambiguously, complain,
     else if some heuristics apply (a space anywhere before first ':')
          then treat as global message,
     else complain (might be a typo-ed intended private message)
  */
  
  cp=strchr(packet->message, ':');

  if (cp != NULL && (cp != &packet->message[0])) {
    Player *pdest=0;
    int nlen, nmatches=0;
    char name[PLAYER_MAX_LENGTH_NAME];
    char *cpblank;

    nlen=MIN(PLAYER_MAX_LENGTH_NAME-1, cp-packet->message);
    strncpy(name, packet->message, nlen);
    name[nlen]='\0';
     
    if( !(pdest=game->find_player_by_name(name)) ) {
      /* no full match,  now look for pre-match */
      for(i=0; i<game->nplayers(); ++i) {
	int j;
	for(j=0; j<nlen; ++j)
	  if(tolower(packet->message[j])!=tolower(game->player(i)->name()[j]))
	    break;
	if(j==nlen) {
	  ++nmatches;
	  pdest=game->player(i);
	}
      }
    }
			   
    if(pdest && nmatches<=1) {
      sprintf(genmsg.message, "->*%s* %s", pdest->name(), cp+1+(*(cp+1)==' '));
      send_packet_generic_message(pplayer->conn(), PACKET_CHAT_MSG, &genmsg);
      sprintf(genmsg.message, "*%s* %s",
	      pplayer->name(), cp+1+(*(cp+1)==' '));
      send_packet_generic_message(pdest->conn(), PACKET_CHAT_MSG, &genmsg);
      return;
    }
    if(nmatches>=2) {
      sprintf(genmsg.message, "Game: %s is an ambiguous name-prefix", name);
      send_packet_generic_message(pplayer->conn(), PACKET_CHAT_MSG, &genmsg);
      return;
    }
    /* Didn't match; check heuristics to see if this is likely
     * to be a global message
     */
    cpblank=strchr(packet->message, ' ');
    if (!cpblank || (cp < cpblank)) {
      sprintf(genmsg.message, "Game: there's no player by the name %s", name);
      send_packet_generic_message(pplayer->conn(), PACKET_CHAT_MSG, &genmsg);
      return;
    }
  }
  /* global message: */
  sprintf(genmsg.message, "<%s> %s", pplayer->name(), packet->message);
  for(i=0; i<game->nplayers(); i++)
    send_packet_generic_message(game->player(i)->conn(), PACKET_CHAT_MSG, 
				&genmsg);
}
