// -*- C++ -*-
/***************************************************************************
                          mem.cpp  -  description
                             -------------------
    begin                : Mon Jun 11 2001
    copyright            : (C) 2001 by Daniel Hermann
    email                : Daniel.Hermann@stud.uni-karlsruhe.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


/**********************************************************************
*
* This module contains memory management functions
*
* TODO: these functions are hardly used and should be abandoned
*
**********************************************************************/ 

#include <stdlib.h>
#include <string.h>

#include "log.h"
#include "mem.h"

/**********************************************************************
 Do whatever we should do when malloc fails.
 At the moment this just prints a log message and calls exit(1)
**********************************************************************/ 
static void handle_alloc_failure(size_t size, const char *called_as,
				 int line, const char *file)
{
  freelog(LOG_FATAL, "Out of memory trying to %s %lu bytes at line %d of %s",
	  called_as, (unsigned long)size, line, file);
  /*
   * Do anything else here (cleanups? safe-save??)
   */
  exit(1);
}

/**********************************************************************
 Function used by cap_malloc macro, malloc() replacement
 No need to check return value.
**********************************************************************/ 
void *cap_real_malloc(size_t size,
		     const char *called_as, int line, const char *file)
{
  void *ptr;
    
  if(size==0) {
    freelog(LOG_NORMAL, "Warning: %s with size %lu at line %d of %s",
	    called_as, (unsigned long)size, line, file);
    return NULL;
  }
  ptr = malloc(size);
  if(ptr==NULL) {
    handle_alloc_failure(size, called_as, line, file);
  }
  return ptr;
}

/**********************************************************************
 Function used by cap_realloc macro, realloc() replacement
 No need to check return value.
**********************************************************************/ 
void *cap_real_realloc(void *ptr, size_t size,
		      const char *called_as, int line, const char *file)
{
  void *new_ptr;
  
  if(ptr==NULL) {
    return cap_real_malloc(size, called_as, line, file);
  }
  if(size==0) {
    freelog(LOG_NORMAL, "Warning: %s with size %lu at line %d of %s",
	    called_as, (unsigned long)size, line, file);
    free(ptr);
    return NULL;
  }
  new_ptr = realloc(ptr, size);
  if(new_ptr==NULL) {
    handle_alloc_failure(size, called_as, line, file);
  }
  return new_ptr;
}

/**********************************************************************
 Function used by cap_calloc macro, calloc() replacement
 No need to check return value.

 I'm pretty sure only the product of nelem and elsize can ever
 matter here, and not their individual values.  (As a matter of C.)
 Except this function doesn't support calloc-ing more memory than
 can be expressing using a single size_t, but that's not likely
 to be a problem.
**********************************************************************/ 
void *cap_real_calloc(size_t nelem, size_t elsize,
		     const char *called_as, int line, const char *file)
{
  size_t size = nelem*elsize;	/* potential overflow */
  void *ptr;
    
  ptr = cap_real_malloc(size, called_as, line, file);
  memset(ptr, 0, size);
  return ptr;
}

/***************************************************************
 Function used by mystrdup macro, strdup() replacement
 No need to check return value.
***************************************************************/
char *real_mystrdup(const char *str, 
		    const char *called_as, int line, const char *file)
{
  char *dest = (char *) cap_real_malloc(strlen(str)+1, called_as, line, file);
  /* no need to check whether dest is non-NULL! */
  strcpy(dest, str);
  return dest;
}

