// -*- C++ -*-
/***************************************************************************
                          packets.h  -  description
                             -------------------
    begin                : Mon Jun 11 2001
    copyright            : (C) 2001 by Daniel Hermann
    email                : Daniel.Hermann@stud.uni-karlsruhe.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef PACKETS_H
#define PACKETS_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "global.h"

#define MAX_PACKET_SIZE   4096
#define NAME_SIZE           10
#define MSG_SIZE          1536
#define ADDR_LENGTH         32

enum packet_type {
  PACKET_REQUEST_JOIN_GAME = 0,
  PACKET_JOIN_GAME_REPLY = 1,
  PACKET_SERVER_SHUTDOWN = 2,
  PACKET_GAME_STATE = 3,

  PACKET_YOUR_TURN = 10,
  PACKET_TURN_DONE = 11,
  PACKET_DICE_ROLL_REQUEST = 12,
  PACKET_DICE_ROLLED = 13,

  PACKET_CARD_DRAWN = 20,

  PACKET_PROPERTY_BUY = 30,
  PACKET_PROPERTY_MORTGAGE = 31,
  PACKET_PROPERTY_UNMORTGAGE = 32,

  PACKET_HOUSES_BUY = 40,
  PACKET_HOUSES_SELL = 41,

  PACKET_JAIL_FREE_REQUEST = 50,
  PACKET_PAY_REQUEST = 51,
  PACKET_DECLARE_BANKRUPT = 52,
  
  PACKET_TRADE = 60,

/*    PACKET_REPORT_REQUEST, */

  // 17
  PACKET_FIELD_INFO = 70,
  PACKET_MONOP_INFO = 71,

  PACKET_PLAYER_INIT_INFO = 80,
  PACKET_PLAYER_INFO = 81,

  // 21
  PACKET_GAME_INIT_INFO = 90,
  PACKET_GAME_INFO = 91,

  PACKET_CHAT_MSG = 100,
  PACKET_PAGE_MSG = 101,
/*    PACKET_SHOW_MESSAGE, */

  PACKET_REMOVE_PLAYER = 110,

  // 26
  PACKET_CONFIRM_ANIM_FINISHED = 120,
  PACKET_ANIM_FINISHED = 121
};

enum report_type {
  REPORT_RICHEST,
  REPORT_SERVER_OPTIONS
};


/*********************************************************
 this structure is a generic packet, which is used by a great
 number of different packets. In general it's used by all
 packets, which only requires a message(apart from the type).
 blah blah..
 used for (e.g.):
  PACKET_SERVER_SHUTDOWN,
  PACKET_CHAT_MSG,
  PACKET_PAGE_MSG,
  PACKET_SHOW_MESSAGE,
and others
*********************************************************/
struct packet_generic_message {
  char message[MSG_SIZE];
  int x,y,event;
};


/*********************************************************
 like the packet above. 
 used for:
  PACKET_GAME_STATE,
  PACKET_YOUR_TURN,
  PACKET_TURN_DONE,
  PACKET_DECLARE_BANKRUPT
  PACKET_DICE_ROLL_REQUEST,
  action packets
and others
*********************************************************/
struct packet_generic_integer {
  int value;
};

/*********************************************************
 ...
 used for:
  ??
*********************************************************/
struct packet_generic_values {
  int id;
  int value1,value2;
};



/*********************************************************
 this packet is the very first packet send by the client.
 the player hasn't been accepted yet.
*********************************************************/
struct packet_req_join_game {
  char name[NAME_SIZE];
  int major_version;
  int minor_version;
  int patch_version;
};


/*********************************************************
 ... and the server replies.
*********************************************************/
struct packet_join_game_reply {
  int you_can_join;             /* true/false */
  char message[MSG_SIZE];
};

/*********************************************************
 dice rolled (sent by server)
*********************************************************/
struct packet_dice_rolled
{
  int playerid;
  int dice1, dice2;
/*    int newloc;		//new location on board */
};

/*********************************************************
 houses buy/sell (sent by client)
*********************************************************/
struct packet_house_request
{
  int monopid;
  int nhouses[MAX_STREETS_IN_MONOP];
};


/* note: no serialize code for some of the packets yet ! (not needed so far) */

/*********************************************************
 information about a player (all static parameters)
*********************************************************/
struct packet_player_init_info
{
  int playerid;
  char name[PLAYER_MAX_LENGTH_NAME];
  char addr[PLAYER_MAX_LENGTH_ADDRESS];
};


/*********************************************************
 information about a player (all variable parameters)
*********************************************************/
struct packet_player_info
{
  int playerid;
  int event;
  int statusflags;	/* i.e. connected, alive, ai etc. */
  int actionflags;	/* i.e. can_declare_bankrupt etc. */

  int location;
  int nturns_in_jail;

  int budget;
  int transaction;	/* amount of money for transaction events */
  int partner;		/* partner in transaction events */
  
  /* TODO: new packet type for player holdings */
  int holdings[BOARD_N_FIELDS];
  int ncards;
  int bank_debts;
  int debts[GAME_MAX_PLAYERS];

};

/*********************************************************
 field information (all static parameters)
*********************************************************/

struct packet_field_init_info
{
  int location;
  int type;
  int price;
  int house_price;
  int rent[STREET_N_RENT_CLASSES];
};


/*********************************************************
 field information (all variable parameters)
*********************************************************/
struct packet_field_info
{
  // TODO: remove location
  int location;
  int event;
  int statusflags;	/* state of the field (i.e. owned, mortgaged, etc.) */
  int actionflags;	/* what can be done with it (i.e can_buy, etc.) */
  int owner;
  int nhouses;
};


/*********************************************************
 monop information
*********************************************************/
struct packet_monop_init_info
{
  int monopid;
  int nstreets;
  int street[MAX_STREETS_IN_MONOP];	/* locations of the streets */
};

struct packet_monop_info
{
  int monopid;
  int event;
  int owner;
};

/*********************************************************
 board information
*********************************************************/
struct packet_board_init_info
{
  struct packet_field_init_info field[BOARD_N_FIELDS];
  struct packet_monop_init_info monop[BOARD_N_MONOPS];

  /* TODO: add chances and community chests information */
};

struct packet_board_info
{
  struct packet_field_info field[BOARD_N_FIELDS];
  struct packet_monop_info monop[BOARD_N_MONOPS];
  /* TODO: add chances and community chests information */
};


/*************************************************************************
 information about all fields and players (all static parameters)
*************************************************************************/
struct packet_game_init_info
{
  struct packet_board_init_info board;

  int nplayers;
  int your_plrid;
  struct packet_player_init_info player[GAME_MAX_PLAYERS];
};

/*********************************************************
 information about all fields (all variable parameters)
*********************************************************/
struct packet_game_info
{
  int event;

  struct packet_board_info board;

  int nplayers;
  struct packet_player_info player[GAME_MAX_PLAYERS];
};

/*********************************************************
 trade information (used in requests and responses)
*********************************************************/

struct packet_trade
{
  int tradeid;

  int state;
  int event;

  int srcid;
  int destid;

  int offer_from;

  int money;
  int prop[BOARD_N_FIELDS];
  int cards;

};




/*********************************************************
this is where the data is first collected, whenever it
arrives to the client/server.
*********************************************************/
struct socket_packet_buffer {
  int ndata;
  int do_buffer_sends;
  unsigned char data[10*MAX_PACKET_SIZE];
};



struct connection {
  int sock, used;
  int first_packet;		/* check byte order on first packet */
  int byte_swap;		/* connection uses non-network byte order */
  char *player; 
  struct socket_packet_buffer buffer;
  struct socket_packet_buffer send_buffer;
  char addr[ADDR_LENGTH];

  enum cmdlevel_id access_level;
  /* Used in the server, "access_level" stores the access granted
   * to the client corresponding to this connection.
   */
};


int read_socket_data(int sock, struct socket_packet_buffer *buffer);
int send_connection_data(struct connection *pc, unsigned char *data, int len);

unsigned char *get_int8(unsigned char *buffer, int *val);
unsigned char *put_int8(unsigned char *buffer, int val);
unsigned char *get_int16(unsigned char *buffer, int *val);
unsigned char *put_int16(unsigned char *buffer, int val);
unsigned char *get_int32(unsigned char *buffer, int *val);
unsigned char *put_int32(unsigned char *buffer, int val);
unsigned char *put_string(unsigned char *buffer, char *mystring);
unsigned char *get_string(unsigned char *buffer, char *mystring);



int send_packet_generic_message(struct connection *pc, int type,
				struct packet_generic_message *message);
struct packet_generic_message *
receive_packet_generic_message(struct connection *pc);


int send_packet_generic_integer(struct connection *pc, int type,
				struct packet_generic_integer *packet);
struct packet_generic_integer *
receive_packet_generic_integer(struct connection *pc);


int send_packet_generic_values(struct connection *pc, int type,
			       struct packet_generic_values *req);
struct packet_generic_values *
receive_packet_generic_values(struct connection *pc);



int send_packet_req_join_game(struct connection *pc, 
			      struct packet_req_join_game *request);
struct packet_req_join_game *
receive_packet_req_join_game(struct connection *pc);


int send_packet_join_game_reply(struct connection *pc, 
			       struct packet_join_game_reply *reply);
struct packet_join_game_reply *
receive_packet_join_game_reply(struct connection *pc);



int send_packet_dice_rolled(struct connection *pc,
			    struct packet_dice_rolled *pack);
struct packet_dice_rolled *
receive_packet_dice_rolled(struct connection *pc);


int send_packet_house_request(struct connection *pc, int type,
			      struct packet_house_request *pack);
struct packet_house_request *
receive_packet_house_request(struct connection *pc);


int send_packet_trade(struct connection *pc,
		      struct packet_trade *pack);
struct packet_trade *
receive_packet_trade(struct connection *pc);


int send_packet_player_init_info(struct connection *pc, 
				 struct packet_player_init_info *pinfo);
struct packet_player_init_info *
receive_packet_player_init_info(struct connection *pc);


int send_packet_player_info(struct connection *pc, 
			    struct packet_player_info *pinfo);
struct packet_player_info *
receive_packet_player_info(struct connection *pc);


int send_packet_field_info(struct connection *pc, 
			   struct packet_field_info *pack);
struct packet_field_info *
receive_packet_field_info(struct connection *pc);

int send_packet_monop_info(struct connection *pc, 
			   struct packet_monop_info *pack);
struct packet_monop_info *
receive_packet_monop_info(struct connection *pc);


int send_packet_game_init_info(struct connection *pc, 
			  struct packet_game_init_info *pinfo);
struct packet_game_init_info *
receive_packet_game_init_info(struct connection *pc);


int send_packet_game_info(struct connection *pc, 
			  struct packet_game_info *pinfo);
struct packet_game_info *
receive_packet_game_info(struct connection *pc);





void *get_packet_from_connection(struct connection *pc, int *ptype);
void remove_packet_from_buffer(struct socket_packet_buffer *buffer);
void flush_connection_send_buffer(struct connection *pc);
void connection_do_buffer(struct connection *pc);
void connection_do_unbuffer(struct connection *pc);

#endif  /* PACKETS_H */
