// -*- C++ -*-
/***************************************************************************
                          board.cpp  -  description
                             -------------------
    begin                : Fre Jun 22 17:19:03 CEST 2001
    copyright            : (C) 2001 by Daniel Hermann
    email                : Daniel.Hermann@stud.uni-karlsruhe.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include "global.h"
#include "events.h"

#include "field.h"
#include "street.h"
#include "railroad.h"
#include "utility.h"
#include "monop.h"

#include "comchest.h"
#include "chance.h"
#include "taxfield.h"
#include "jailfield.h"

#include "board.h"

struct monop_struct {
  int street[MAX_STREETS_IN_MONOP];
};

struct monop_struct monop_init[BOARD_N_MONOPS] = 
#include "monops.dat"
;

struct field_struct {
  int type;
  int price;
  int house_price;
  int rent[STREET_N_RENT_CLASSES];
};

struct field_struct field_init[BOARD_N_FIELDS] =
#include "fields.dat"
;				 

// TODO: configurable initialization on console 
Board::Board()
{
  int loc, no;
  
  // create Fields
  
  for (loc=0; loc<BOARD_N_FIELDS; ++loc) {
    switch (field_init[loc].type) {

    case STREET:
      _field[loc] = new Street(loc, 
			       field_init[loc].price, 
			       field_init[loc].house_price,
			       field_init[loc].rent);
      break;

    case RAILROAD:
      _field[loc] = new Railroad(loc, 
				 field_init[loc].price,
				 field_init[loc].rent[0]);
      break;

    case UTILITY:
      _field[loc] = new Utility(loc, 
				field_init[loc].price,
				field_init[loc].rent);
      break;

    case COMCHEST:
      _field[loc] = new Comchest(loc); 
      break;

    case CHANCE:
      _field[loc] = new Chance(loc); 
      break;

    case TAXFIELD:
      _field[loc] = new Taxfield(loc);
      break;

    case JAILFIELD:
      _field[loc] = new Jailfield(loc);
      break;

    case SAFEFIELD: 
      _field[loc] = new Field(loc); 
      break;

    default:
      assert(0);	// bug: no other types possible
    }
  }

  // create Monops
  
  for (no=0; no<BOARD_N_MONOPS; no++) {
    _monop[no] = new Monop(no, _field, monop_init[no].street);
  }    

}

Board::~Board()
{
  int i;

  for (i=0; i<BOARD_N_FIELDS; i++)
    delete _field[i];
  for (i=0; i<BOARD_N_MONOPS; i++)
    delete _monop[i];
}


/*
 * fill info packets with data
 */

void Board::init_info_for(Player *player, struct packet_board_init_info *pack)
{
  int i;

  for (i=0; i<BOARD_N_FIELDS; ++i) 
    _field[i]->init_info_for(player, &pack->field[i]);
    
  for (i=0; i<BOARD_N_MONOPS; ++i)
    _monop[i]->init_info_for(player, &pack->monop[i]);
  
}


void Board::info_for(Player *player, struct packet_board_info *pack)
{
  int i;

  for (i=0; i<BOARD_N_FIELDS; ++i)
    _field[i]->info_for(player, &pack->field[i], E_NOEVENT);

  for (i=0; i<BOARD_N_MONOPS; ++i)
    _monop[i]->info_for(player, &pack->monop[i], E_NOEVENT);
  
}


