// -*- C++ -*-
/***************************************************************************
                          board.h  -  description
                             -------------------
    begin                : Fre Jun 22 17:19:03 CEST 2001
    copyright            : (C) 2001 by Daniel Hermann
    email                : Daniel.Hermann@stud.uni-karlsruhe.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef BOARD_H
#define BOARD_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "global.h"
#include "packets.h"

class Player;

enum locations {
  GO = 0,

  JUST_VISITING = 10,
  FREE_PARKING = 20,
  GOTO_JAIL = 30,
  JAIL = 40,

  // Tax fields
  INCOME_TAX = 4,
  LUXURY_TAX = 38,

  // chances and comchests (not used yet!)
  // note: don't use these, because they fix the board
//    CHANCE_1 = 7,
//    CHANCE_2 = 22,
//    CHANCE_3 = 36,
//    COMCHEST_1 = 2,
//    COMCHEST_2 = 17,
//    COMCHEST_3 = 33,

  // railroads (in german version south, west, north and central)
  RRSOUTH = 5,
  RRWEST = 15,
  RRNORTH = 25,
  RRCENTRAL = 35,

  // important streets (for cards)
  ST_CHARLES_PLACE = 11,
  ILLINOIS_AVE = 24,
  BOARDWALK = 39,

  // utilities
  UELECTRIC = 12,
  UWATER = 28,

  // TODO: change BOARD_N_FIELDS into 41, so that JAIL becomes a valid 
  // field id. Reduces risk of segfaults/assertions  ???
  // On the other hand, the BOARD does have only 40 fields !
  LAST_LOC = BOARD_N_FIELDS + 1	// includes JAIL, used in Client

};

class Field;
class Monop;

class Board {
  
public:

  /* 
   * server board constructor (TODO: use config file and console configuration)
   */
  Board();

  /*
   * client board constructor
   */  
  Board(struct packet_board_init_info *);

  /*
   * destructor
   */
  ~Board();


  /* should use const specifiers, but this caused problems */
  Field * const field(int id) const { return _field[id]; }
  Monop * const monop(int id) const { return _monop[id]; }

  /*
   * generate info (server side)
   */
  void init_info_for(Player *, struct packet_board_init_info *);
  void info_for(Player *, struct packet_board_info *);

  /*
   * update data  (client side)
   */
  void update(struct packet_board_info *);

protected:

  Field *_field[BOARD_N_FIELDS];
  Monop *_monop[BOARD_N_MONOPS];

};

#endif /* BOARD_H */


