// -*- C++ -*-
/***************************************************************************
                          cardstack.cpp  -  description
                             -------------------
    begin                : Fre Jun 22 17:19:03 CEST 2001
    copyright            : (C) 2001 by Daniel Hermann
    email                : Daniel.Hermann@stud.uni-karlsruhe.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <stdlib.h>
#include <assert.h>
#include <time.h>

#include "log.h"	// only debugging


#include "cardstack.h"

//
// some details:
// - the cardstack size stays fixed
// - shuffle() creates a representation of all cards randomly ordered
//   and stores it in the 'shuffled' array. All card numbers only appear
//   once in this
// - cards can be removed (e.g. when drawing a jail card) and put back
// - the 'removed' array contains a bool for every card whether it was removed
//   or is available (it gets cleared when put_back() is called)
//

Cardstack::Cardstack(int ncards)
  : randseed(time(NULL)),
    _ncards(ncards)
{
  srand(randseed); // previously done by Game::Game, not needed here ??

  assert(ncards>0);

  shuffled = new int[ncards];
  shuffle();

  removed = new bool[ncards];
  for (int i=0; i<ncards; ++i) removed[i] = false;

}

Cardstack::~Cardstack()
{
  delete [] shuffled;
  delete [] removed;
}  

void Cardstack::shuffle()
{
  int i;
  bool used[_ncards];

  for (i=0; i<_ncards; ++i)
    used[i] = false;

  for (i=0; i<_ncards; ++i) {
    do {
      shuffled[i] = (int) ((double) _ncards * rand()
			   / (RAND_MAX + 1.0));
    } while (used[shuffled[i]]);
    used[shuffled[i]] = true;
  }

  freelog(LOG_DEBUG, "shuffled:");
  for (i=0; i<_ncards; ++i)
    freelog(LOG_DEBUG, "card %2d: shuffled=%2d", i, shuffled[i]);

}

int Cardstack::draw_card()
{
  static int counter = 0;
  int no_more_cards = 0;  // to detect infinite loops

  int card_no;

  // search first non-removed card on the shuffled card deck
  do {
    if (counter>=_ncards) {	// all cards were used
      assert(no_more_cards==0);  // this happens when all cards were removed
      shuffle();
      counter = 0;
      no_more_cards=1;
    }
    card_no = shuffled[counter++];
  } while(removed[card_no]);

  return card_no;
}

void Cardstack::remove(int card_no)
{
  if (card_no>=0 && card_no<_ncards)
    removed[card_no] = true;
}

void Cardstack::put_back(int card_no)
{
  if (card_no>=0 && card_no<_ncards)
    removed[card_no] = false;
}


void JailCard::put_back()
{
  _stack->put_back(_card_no);
}


