// -*- C++ -*-
/***************************************************************************
                          chance.cpp  -  description
                             -------------------
    begin                : Fre Jun 22 17:19:03 CEST 2001
    copyright            : (C) 2001 by Daniel Hermann
    email                : Daniel.Hermann@stud.uni-karlsruhe.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <stdlib.h>
#include <assert.h>
#include <time.h>

#include "log.h"
#include "packets.h"
#include "events.h"

#include "game.h"
#include "board.h"
#include "player.h"

#include "cardstack.h"

#include "chance.h"

extern Game *game;


enum card_type {
  CARD0 = 0, CARD1, CARD2, CARD3, CARD4, CARD5, CARD6, CARD7, CARD8, CARD9, 
  CARD10, CARD11, CARD12, CARD13, CARD14, CARD15, 
  CARD_LAST
};



// static attribute: card stack

Cardstack *Chance::stack = NULL;


Chance::Chance(const int location)
  : Field(location)
{
  _type = CHANCE;

  if (stack == NULL) {
    stack = new Cardstack(GAME_N_CHANCES);
  }
}

Chance::~Chance()
{
  _type = CHANCE;

  if (stack != NULL) delete stack;
}

void Chance::enter(Player *plr)
{
  int newloc;
  int card_no = stack->draw_card();

  game->send_card_drawn(NULL, plr, CHANCE, card_no);

  freelog(LOG_NORMAL, ("Player has entered chance field %d and draws "
		       "a card: %d"), 
	  _location, card_no);
  
  switch (card_no) {

  case CARD0:	// "get out of jail free" card
    plr->add_jail_card(new JailCard(stack, card_no));
    stack->remove(card_no);
    break;

  case CARD1:
  case CARD4:
    // this isn't safe, if board data changes ?!!!
    if (_location < RRSOUTH) newloc = RRSOUTH;
    else if (_location < RRWEST) newloc = RRWEST;
    else if (_location < RRNORTH) newloc = RRNORTH;
    else if (_location < RRCENTRAL) newloc = RRCENTRAL;
    else newloc = RRSOUTH;	// fields around Board Walk
    plr->move_to(newloc, PASS_GO);
    break;
  case CARD2:
    if (_location > UELECTRIC && _location < UWATER) newloc = UWATER;
    else newloc = UELECTRIC;
    plr->move_to(newloc, PASS_GO);
    break;
  case CARD3:
    plr->move(-3, DONT_PASS_GO);
    break;
  case CARD5:
    plr->goto_jail(E_CARD_GOTO_JAIL);
    break;
  case CARD6:
    plr->move_to(RRSOUTH, PASS_GO);
    break;
  case CARD7:
    plr->move_to(BOARDWALK, PASS_GO);
    break;
  case CARD8:
    plr->move_to(ILLINOIS_AVE, PASS_GO);
    break;
  case CARD9:
    plr->move_to(GO, PASS_GO);
    break;
  case CARD10:
    plr->move_to(ST_CHARLES_PLACE, PASS_GO);
    break;
  case CARD11:
    // street repairs
    plr->renovate_buildings(25, 100);
    break;
  case CARD12:
    plr->pay_to_all(50, E_CARD_PAY_TO_ALL);
    break;
  case CARD13:
    plr->pay(NULL, 15, E_CARD_PAY);
    break;
  case CARD14:
    plr->earn(NULL, 50, E_CARD_COLLECT);
    break;
  case CARD15:
    plr->earn(NULL, 150, E_CARD_COLLECT);
    break;
    
  default:
    assert(CARD_LAST == GAME_N_CHANCES);
    assert(card_no < CARD_LAST);

  }
}

