// -*- C++ -*-
/***************************************************************************
                          comchest.cpp  -  description
                             -------------------
    begin                : Fre Jun 22 17:19:03 CEST 2001
    copyright            : (C) 2001 by Daniel Hermann
    email                : Daniel.Hermann@stud.uni-karlsruhe.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <stdlib.h>
#include <assert.h>
#include <time.h>

#include "log.h"
#include "packets.h"
#include "events.h"

#include "game.h"
#include "board.h"
#include "player.h"

#include "cardstack.h"

#include "comchest.h"

extern Game *game;


enum card_type {
  CARD0 = 0, CARD1, CARD2, CARD3, CARD4, CARD5, CARD6, CARD7, CARD8, CARD9, 
  CARD10, CARD11, CARD12, CARD13, CARD14, CARD15, 
  CARD_LAST
};



// static attribute: card stack

Cardstack *Comchest::stack = NULL;


Comchest::Comchest(const int location)
  : Field(location)
{
  _type = COMCHEST;

  if (stack == NULL) {
    stack = new Cardstack(GAME_N_COMCHESTS);
  }
}

Comchest::~Comchest()
{
  if (stack != NULL) delete stack;
}

void Comchest::enter(Player *plr)
{
  int card_no = stack->draw_card();

  game->send_card_drawn(NULL, plr, COMCHEST, card_no);

  freelog(LOG_NORMAL, ("Player has entered community chest %d and draws "
		       "a card: %d"), 
	  _location, card_no);
  
  switch (card_no) {

  case CARD0:	// "get out of jail free" card
    plr->add_jail_card(new JailCard(stack, card_no));
    stack->remove(card_no);
    break;

  case CARD1:
    plr->earn(NULL, 25, E_CARD_COLLECT);
    break;
  case CARD2:
    plr->earn(NULL, 200, E_CARD_COLLECT);
    break;
  case CARD3:
    plr->earn(NULL, 20, E_CARD_COLLECT);
    break;
  case CARD4:
    plr->pay(NULL, 100, E_CARD_PAY);
    break;
  case CARD5:
    plr->earn(NULL, 100, E_CARD_COLLECT);
    break;
  case CARD6:
    plr->earn(NULL, 45, E_CARD_COLLECT);
    break;
  case CARD7:
    // street repairs
    plr->renovate_buildings(40, 115);
    break;
  case CARD8:
    plr->earn(NULL, 100, E_CARD_COLLECT);
    break;
  case CARD9:
    plr->earn(NULL, 11, E_CARD_COLLECT);
    break;
  case CARD10:
    plr->move_to(GO, PASS_GO);
    break;
  case CARD11:
    plr->earn(NULL, 100, E_CARD_COLLECT);
    break;
  case CARD12:
    plr->pay(NULL, 150, E_CARD_PAY);
    break;
  case CARD13:
    plr->goto_jail(E_CARD_GOTO_JAIL);
    break;
  case CARD14:
    plr->earn_from_all(50, E_CARD_COLLECT_FROM_ALL);
    break;
  case CARD15:
    plr->pay(NULL, 50, E_CARD_PAY);
    break;
    
  default:
    assert(CARD_LAST == GAME_N_COMCHESTS);
    assert(card_no < CARD_LAST);

  }
}

