// -*- C++ -*-
/***************************************************************************
                          field.h  -  description
                             -------------------
    begin                : Fre Jun 22 17:19:03 CEST 2001
    copyright            : (C) 2001 by Daniel Hermann
    email                : Daniel.Hermann@stud.uni-karlsruhe.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef FIELD_H
#define FIELD_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "global.h"
#include "packets.h"

enum field_type {
  SAFEFIELD = 0,	/* safe spot			*/

  STREET,		/* normal property		*/
  RAILROAD,		/* railroad			*/
  UTILITY,		/* water works - electric co	*/

  COMCHEST,		/* community chest		*/
  CHANCE,		/* chance (surprise!!!)		*/

  TAXFIELD,		/* tax field			*/

  JAILFIELD		/* Go To Jail and Just visiting */
};

enum field_statusflags {
  IS_OWNED = 1,
  IS_YOURS = 2,
  IS_MORTGAGED = 4,
  IS_MONOP = 8,
};

enum field_actionflags {
  CAN_BUY = 1,

  CAN_MORTGAGE = 2,
  CAN_UNMORTGAGE = 4,

  CAN_BUILD_HOUSES = 8,
  CAN_SELL_HOUSES = 16,
};


/* need undefined state for int values */
#define UNDEFINED  -1

class Player;

class Field {
  
public:
  
  Field(const int location);

  Field(struct packet_field_init_info *);

  virtual ~Field();

  const int location() const { return _location; }

  enum field_type type() const { return _type; }

  virtual void enter(Player *);

  virtual int const status(const Player *) const;
  virtual int const actions(const Player *) const;

  virtual void init_info_for(const Player *, struct packet_field_init_info *);
  virtual void info_for(const Player *, struct packet_field_info *, int event);

protected:
  
  int _location;
  enum field_type _type;
  
};

#endif /* FIELD_H */
