// -*- C++ -*-
/***************************************************************************
                          game.cpp  -  description
                             -------------------
    begin                : Fre Jun 22 17:19:03 CEST 2001
    copyright            : (C) 2001 by Daniel Hermann
    email                : Daniel.Hermann@stud.uni-karlsruhe.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <assert.h>

#include "global.h"
#include "mem.h"
#include "shared.h"
#include "packets.h"
#include "events.h"

#include "main.h"
#include "sernet.h"

#include "log.h"
#include "console.h"
#include "plrhand.h"

#include "board.h"
#include "field.h"
#include "monop.h"
#include "property.h"
#include "comchest.h"
#include "player.h"

#include "game.h"

/*
 * TODO: - improve freelogs (LOG_VERBOSE??)
 *       - only one value in trade prop-arrays (see Trade.cc)
 */

extern enum server_states server_state;
extern int force_end_of_sniff;

/***************************************************************
...
***************************************************************/
Game::Game()
  : _board(0),
    _nplayers(0),
    _startbudget(GAME_STARTBUDGET),
    _income_on_GO(GAME_INCOME_ON_GO),
    _curr_plr(-1),
    _rolled_dice(false),
    _ndoubles(0),
    _turn_done(false)
{
  int i,j;

  for (i=0; i<GAME_MAX_PLAYERS; ++i) {
    _player[i] = 0;
    for (j=0; j<=i; ++j)
      _trade[i][j] = _trade[j][i] = 0;
    _plr_anim_finished[i] = true;
  }

  /* We strip the high bit for now because ... ? */
  _randseed = time(0) & (MAX_UINT32 >> 1);
  mysrand(_randseed);
  srand(_randseed);

  _last_dice[0] = _last_dice[1] = 0;
    
}

/***************************************************************
...
***************************************************************/
Game::~Game()
{
  int i,j;

  for (i=0; i<GAME_MAX_PLAYERS; ++i) {
    if (_player[i]) delete _player[i];
    for (j=0; j<=i; ++j) {
      if (_trade[i][j]) delete _trade[i][j];
      if (_trade[j][i]) delete _trade[j][i];
    }
  }
  
  if (_board) delete _board;

}

const bool Game::anim_in_progress() const
{
  bool all_anims_finished = true;

  for (int i=0; i<_nplayers; ++i) {
    if (_player[i]->is_connected() && !_plr_anim_finished[i]) 
      all_anims_finished = false;
  }

  return !all_anims_finished;
}

void Game::set_anim_finished(const int plrid, bool finished)
{
  if (!nobody(plrid)) {
    _plr_anim_finished[plrid] = finished;
  }
}

/*
 * adds a new Player to the player array
 */
int Game::add_player(Player *plr) 
{
  if (_nplayers < GAME_MAX_PLAYERS) {
    _player[_nplayers] = plr;
    _nplayers++;
    return 0;
  } else return 1;
}

  /* 
   * TODO: Spieler lschen und brige Spieler neu anordnen
   * im Moment ist das Lschen eines Spielers so nicht mglich !!!!
   */
int Game::remove_player(const int plrno)
{
  int i;
  Player *pplayer=_player[plrno];

  if (pplayer == 0) return 1;

  switch (server_state) {

  case PRE_GAME_STATE: case SELECT_TOKEN_STATE:
    delete pplayer;
    for (i=plrno; i<_nplayers-1; ++i) {
      _player[i] = _player[i+1];  // shift all players down
    }
    _nplayers--;
    break;

  case RUN_GAME_STATE:    
    // no implementation for removing player in a running game yet !
    // TODO: implement it, with all consequences
//      pplayer->cleanup_all_holdings(); etc...
    break;

  default:
    break;
  }
  return 0;
}

/***************************************************************
...
***************************************************************/
int Game::remove_all_players()
{
  int i;

  for(i=0; i<_nplayers; ++i)
    remove_player(i);
  _nplayers=0;

  return 0;
}

/***************************************************************
...
***************************************************************/
Player * Game::find_player_by_name(char *name)
{
  int i;

  for (i=0; i<_nplayers; ++i)
    if(!mystrcasecmp(name, const_cast<char *>(_player[i]->name())))
      return _player[i];
  
  return 0;
}


/*
 * initialize the Board, unique ID's for players, 
 * fix all options, rulesets etc. 
 *
 */
int Game::run()
{
  int i;

  _board = new Board();
  for (i=0; i<_nplayers; ++i) {
    _player[i]->set_id(i);
  }

  send_all_info(TOALL);

  return 0;
}



/*
 * fill info packets with data
 */

void Game::init_info_for(Player *plr, struct packet_game_init_info *pack) const
{
  int i;

  _board->init_info_for(plr, &pack->board);
  
  pack->nplayers = _nplayers;
  pack->your_plrid = plr->id();
  
  for (i=0; i<_nplayers; ++i)
    _player[i]->init_info_for(plr, &pack->player[i]);
}


void Game::info_for(Player *plr, struct packet_game_info *pack) const
{
  int i;

  pack->event = E_NOEVENT;

  _board->info_for(plr, &pack->board);
  
  pack->nplayers = _nplayers;
  
  for (i=0; i<_nplayers; ++i)
    _player[i]->info_for(plr, &pack->player[i], E_NOEVENT);
}

int Game::check_trade_packet_consistency(Player *plr, 
					 struct packet_trade *pack)
{
#define ERR_EVENT	1
#define ERR_DEST	2
#define ERR_PROP	3
#define IS_SRC		IS_YOURS

  int i;

//    int tradeid = pack->tradeid;	// not used yet

  // tradeid's should be equal ( in future )
//    if (trade) {
//      assert(trade->id() == tradeid);
//    }

  int event = pack->event;
  int destid = pack->destid;
//    int money = pack->money;
  bool is_src;	// is plr the src or dest of trade

  Player *other;
  Player *src;
  Player *dest;
  Trade *trade;

  if (event!=E_TRADE_OFFER && event!=E_TRADE_REPLY &&
      event!=E_TRADE_AGREE && event!=E_TRADE_REJECT) {
    notify_player(plr, "Error: trade packet with no trade event");
    freelog(LOG_NORMAL, "error: trade packet with no trade event");
    return ERR_EVENT;
  }    
    
  if (nobody(destid)) {
    notify_player(plr, "Game: dest player id in trade packet not valid");
    freelog(LOG_NORMAL, "error: dest player id in trade packet not valid");
    return ERR_DEST;
  } 

  if (destid == plr->id()) {	// no trade with yourself!
    notify_player(plr, "Game: you can't have trade with yourself");
    freelog(LOG_NORMAL, "error: can't have trade with himself");
    return ERR_DEST;
  } 
    
  // here: destid ok

  dest = _player[destid];

  if (!dest->is_alive()) {	// don't have trade with dead players!
    notify_player(plr, "Game: you can't have trade with a dead player %s.",
		  dest->name());
    freelog(LOG_NORMAL, "error: can't have trade with dead player %s",
	    dest->name());
    return ERR_DEST;
  }

  // here: dest ok

  trade = get_trade(plr, dest);

  if (event == E_TRADE_OFFER) {
    // an offer is only valid, if there isn't any trade between the players yet
    if (trade != 0) {
      notify_player(plr, "Game: you already have trade with %s, try 'reply'",
		    dest->name());
      freelog(LOG_NORMAL, "error: already has trade with %s", dest->name());
      return ERR_EVENT;
    }      
  } else {	// no offer but reply, agree or reject
    // there should be an existing Trade object
    if (trade == 0) {
      notify_player(plr, "Game: you don't have trade with %s yet, try 'offer'",
		    dest->name());
      freelog(LOG_NORMAL, "error: hasn't trade with %s yet", dest->name());
      return ERR_EVENT;
    }
    assert(dest->get_trade_with(plr) == trade);	// plr and dest share trade obj
  }

  if (event == E_TRADE_AGREE || event == E_TRADE_REJECT)
    return 0;	// all needed info for these events is correct


  // for OFFER and REPLY, we must also check the property array

  // get the roles of plr and dest in trade (one should be src, the other dest)
  
  if (event==E_TRADE_REPLY) {
    src = trade->get_src();
    dest = trade->get_dest();
    is_src = (src == plr);
    if (is_src) other = dest;
    else other = src;
  } else { // E_TRADE_OFFER
    src = plr;
    other = dest;
    is_src = true;
  }

  for (i=0; i<BOARD_N_FIELDS; ++i) {
    Property *p = dynamic_cast<Property *>(_board->field(i));

    if (pack->prop[i] == T_NOTRADE) { // field not included in trade
      // always ok
      continue;
    } else { // field/property involved in trade
      // check if it is a property
      if (!p) {
	notify_player(plr, "Game: field %d is no property", i);
	freelog(LOG_NORMAL, "error: field %d is no property", i);
	return ERR_PROP;
      }
      // change (!!!) prop[i] of the packet, fill it with statusflag
      pack->prop[i] = p->status(src);
      freelog(LOG_VERBOSE, "trade packet rewritten: prop[%d] = %d", 
	      i, pack->prop[i]);
      
      // IS_YOURS flag has alias IS_SRC here
      // 2 possibilities: 
      //    IS_SRC set, we are sure that property belongs to src
      //    IS_SRC not set, test if property belongs to dest, else error
      if (pack->prop[i] & IS_SRC)
      if (!(pack->prop[i] & IS_SRC)) { // test if IS_DEST
	if (p->owner() != dest) {
	  notify_player(plr, "Game: property %d is not owned by %s", 
			i, dest->name());
	  freelog(LOG_NORMAL, "error: property %d not owned by %s", 
		  i, dest->name());
	  return ERR_PROP;
	}
      }
    }
  }
  // check if there are no properties involved
  // it's not allowed to trade only money

  int nprops = 0;
  for (i=0; i<BOARD_N_FIELDS; ++i) {
    if (pack->prop[i] != T_NOTRADE) nprops++;
  }
  
  if (nprops == 0) {
    notify_player(plr, "Game: no properties involved in your trade");
    freelog(LOG_NORMAL, "error: no properties involved in trade");
    return ERR_PROP;
  }

    
//      default:

//        notify_player(plr, "Error: invalid trade packet, unknown "
//  		    "value in prop[%d]: %d", i, pack->prop[i]);
//        freelog(LOG_NORMAL, "error: invalid trade packet from %s, unknown "
//  	      "value in prop[%d]: %d", plr->name(), i, pack->prop[i]);
//        return ERR_PROP;
//      }

  return 0;	// packet is OK
    
#undef ERR_EVENT
#undef ERR_DEST
#undef ERR_PROP
#undef IS_SRC
}

/*
 * network input handler
 */

void Game::handle_turn_done(Player *plr)
{
  freelog(LOG_VERBOSE, "turn done packet from %s", plr->name());
  
  if (plr != _player[_curr_plr]) {
    notify_player(plr, "Game: It's not your turn, wait for %s",
		  _player[_curr_plr]->name());
    freelog(LOG_NORMAL, "error: not his turn");
    return;
  } else if (!_rolled_dice) {
    notify_player(plr, "Game: You didn't dice yet.");
    freelog(LOG_NORMAL, "error: didn't dice yet");
    return;
  } 

  
  else if (plr->settle_debts() != 0) { // most likely, player isn't afloat
    notify_player(plr,("Game: You can't end turn with debts.\n"
		       "Game: Try selling houses, mortgaging properties etc.\n"
			"Game: or declare yourself bankrupt") );
    freelog(LOG_NORMAL, "error: turn_done not allowed: not afloat");
    return;
  } 
  
  
  // OK
  freelog(LOG_NORMAL, "%s has ended turn", plr->name());

  force_end_of_sniff = 1;
  // see main.cc: -> next_turn() is initiated
}

void Game::handle_dice_roll_request(Player *plr, 
				    struct packet_generic_integer *pack)
{
  int newloc, nsteps;
  int dice_sum = pack->value;

  freelog(LOG_NORMAL, "dice roll request from %s", plr->name());

  if (_curr_plr != plr->id()) {
    notify_player(plr, "Game: It's not your turn, wait for %s", 
		  _player[_curr_plr]->name());
    freelog(LOG_NORMAL, "error: not his turn");
    return;
  } 
  if (_rolled_dice) {
    notify_player(plr, "Game: You already rolled the dice");
    freelog(LOG_NORMAL, "error: has already rolled dice");
    return;
  } 
 
  // OK ( here: plr == _player[curr_plr] )

  buffer_all_connections();
//    send_anim(TOALL, PACKET_ANIM_BEGIN);

  roll_dice(0, dice_sum);
  
  freelog(LOG_NORMAL, "%s rolled dice: dice1=%d, dice2=%d", plr->name(), 
	  _last_dice[0], _last_dice[1]);

  if (plr->is_in_jail()) {

    if (_ndoubles > 0) { // rolled doubles -> get free
      plr->leave_jail(E_LEAVE_JAIL_ROLLED_DOUBLES);
      freelog(LOG_NORMAL, "%s leaves jail after throwing doubles", 
	      plr->name());
      goto move_now;
    } else if (plr->nturns_in_jail() == 3) { 
      // after 3 turns in jail, pay fine and leave jail
      plr->leave_jail(E_LEAVE_JAIL_NORMAL);
      freelog(LOG_NORMAL, "%s leaves jail after 3 turns", plr->name());
      goto move_now;
    } else {	// stay in jail
      plr->stay_in_jail();
      freelog(LOG_NORMAL, "%s stays in jail", plr->name());
      goto dont_move;
    }

  } else {	// player not in jail
    
    if (_ndoubles >= 3) {	// rolled 3 doubles -> goto jail
      plr->goto_jail(E_GOTO_JAIL_ROLLED_DOUBLES);
      freelog(LOG_NORMAL, "%s must go to jail after 3 doubles", 
	      plr->name());
      goto dont_move;
    } else {
      // normal move, no jail events
    }
  }

 move_now:
  nsteps = _last_dice[0] + _last_dice[1];
  freelog(LOG_NORMAL, "%s moves %d steps", plr->name(), nsteps); 
  newloc = plr->move(nsteps, PASS_GO);

 dont_move:

//    send_anim(TOALL, PACKET_ANIM_END);
  send_anim(TOALL, PACKET_CONFIRM_ANIM_FINISHED);
  unbuffer_all_connections();
  wait_for_anim_finished();
  force_end_of_sniff = 1;
}

void Game::handle_property_buy(Player *plr, 
			       struct packet_generic_integer *pack)
{
  assert(plr);

  int loc = pack->value;
  Property *prop;
  
  freelog(LOG_NORMAL, "property buy request from %s: loc=%d", 
	  plr->name(), loc);

  if (_curr_plr != plr->id()) {
    notify_player(plr, "Game: It's not your turn, wait for %s",
		  _player[_curr_plr]->name());
    freelog(LOG_NORMAL, "error: not his turn");
    return;
  } 
  if (! ( (loc >= 0 && loc < BOARD_N_FIELDS) || loc == JAIL ) ) {
    notify_player(plr, "Game: field %d nonexistent", loc);
    freelog(LOG_NORMAL, "error: field %d nonexistent", loc);
    return;
  }
  if (loc != plr->location()) {		// plr is not on the field
    notify_player(plr, "Game: You're not on field %d",
		  loc);
    freelog(LOG_NORMAL, "error: is not on that field");
    // for debugging purposes do not require to be on a field to buy it
    if (!CHEATING_ALLOWED) return;  
    freelog(LOG_NORMAL, "plr %s is cheating!", plr->name());
    notify_player(TOALL, "Game: %s is cheating!", plr->name());
  } 
  if (loc == JAIL) {	// cannot buy jail :-)
    notify_player(plr, "Game: You can't buy jail");
    freelog(LOG_NORMAL, "error: can't buy jail");
    return;
  }
  if (plr->debts_sum() > 0) {   // first settle debts before buying anything
    notify_player(plr, "Game: You are in debts. Please settle them first");
    freelog(LOG_NORMAL, "error: is in debts");
  }

  prop = dynamic_cast<Property *>(_board->field(loc));

  if (prop == 0) {	// it's no property
    notify_player(plr, "Game: field %d is no property", loc);
    freelog(LOG_NORMAL, "error: field is no property");
    return;
  } 
  
  // OK (further conditions in prop->buy)

  buffer_all_connections();

  int errcode = prop->buy(plr);
  
  unbuffer_all_connections();

  if (errcode == 0)
    freelog(LOG_NORMAL, "property %d bought by %s", loc, plr->name());

}    

void Game::handle_property_mortgage(Player *plr, 
				    struct packet_generic_integer *pack)
{
  assert(plr);

  int loc = pack->value;
  Property *prop;
  
  assert((loc >= 0 && loc < BOARD_N_FIELDS) || loc == JAIL);

  freelog(LOG_NORMAL, "property mortgage request from %s: loc=%d", 
	  plr->name(), loc);

  if (_curr_plr != plr->id()) {
    notify_player(plr, "Game: It's not your turn, wait for %s",
		  _player[_curr_plr]->name());
    freelog(LOG_NORMAL, "error: not his turn");
    return;
  } 
  if (loc == JAIL) {	// cannot mortgage jail :-)
    notify_player(plr, "Game: You can't mortgage jail");
    freelog(LOG_NORMAL, "error: can't buy jail");
    return;
  }

  prop = dynamic_cast<Property *>(_board->field(loc));

  if (prop == 0) {	// it's no property
    notify_player(plr, "Game: field %d is no property", loc);
    freelog(LOG_NORMAL, "error: field is no property");
    return;
  } 
  
  // OK (further conditions in prop->mortgage)

  buffer_all_connections();

  int errcode = prop->mortgage(plr);
  
  unbuffer_all_connections();

  if (errcode == 0)
    freelog(LOG_NORMAL, "property %d mortgaged by %s", loc, plr->name());

}    


void Game::handle_property_unmortgage(Player *plr, 
				    struct packet_generic_integer *pack)
{
  assert(plr);

  int loc = pack->value;
  Property *prop;
  
  assert(loc >= 0 && loc < BOARD_N_FIELDS);

  freelog(LOG_NORMAL, "property unmortgage request from %s: loc=%d", 
	  plr->name(), loc);

  if (_curr_plr != plr->id()) {
    notify_player(plr, "Game: It's not your turn, wait for %s",
		  _player[_curr_plr]->name());
    freelog(LOG_NORMAL, "error: not his turn");
    return;
  } 
  if (loc == JAIL) {	// cannot unmortgage jail :-)
    notify_player(plr, "Game: You can't unmortgage jail");
    freelog(LOG_NORMAL, "error: can't buy jail");
    return;
  }

  prop = dynamic_cast<Property *>(_board->field(loc));

  if (prop == 0) {	// it's no property
    notify_player(plr, "Game: field %d is no property", loc);
    freelog(LOG_NORMAL, "error: field is no property");
    return;
  } 
  
  // OK (further conditions in prop->unmortgage)

  buffer_all_connections();

  int errcode = prop->unmortgage(plr);
  
  unbuffer_all_connections();

  if (errcode == 0)
    freelog(LOG_NORMAL, "property %d unmortgaged by %s", loc, plr->name());

}    


void Game::handle_houses_buy(Player *plr, 
			     struct packet_house_request *pack)
{
  assert(plr);

  int monid = pack->monopid;
  int nhouses[] = { pack->nhouses[0], pack->nhouses[1], pack->nhouses[2] };

  assert(monid >= 0 && monid < BOARD_N_MONOPS);

  Monop *monop = _board->monop(monid);
  
  freelog(LOG_NORMAL, "houses buy request from %s: monid=%d", 
	  plr->name(), monid);

  if (_curr_plr != plr->id()) {
    notify_player(plr, "Game: It's not your turn, wait for %s",
		  _player[_curr_plr]->name());
    freelog(LOG_NORMAL, "error: not his turn");
    return;
  }
  
  // OK (further conditions in monop->houses_buy)

  buffer_all_connections();

  int errcode = monop->buy_houses(plr, nhouses);
  
  unbuffer_all_connections();

  if (errcode == 0)
    freelog(LOG_NORMAL, "houses bought in monop %d by %s", monid, plr->name());

}    

void Game::handle_houses_sell(Player *plr, 
			      struct packet_house_request *pack)
{
  assert(plr);

  int monid = pack->monopid;
  int nhouses[] = { pack->nhouses[0], pack->nhouses[1], pack->nhouses[2] };

  assert(monid >= 0 && monid < BOARD_N_MONOPS);

  Monop *monop = _board->monop(monid);
  
  freelog(LOG_NORMAL, "houses sell request from %s: monid=%d", 
	  plr->name(), monid);

  if (_curr_plr != plr->id()) {
    notify_player(plr, "Game: It's not your turn, wait for %s",
		  _player[_curr_plr]->name());
    freelog(LOG_NORMAL, "error: not his turn");
    return;
  } 

  // OK (further conditions in monop->houses_buy)

  buffer_all_connections();

  int errcode = monop->sell_houses(plr, nhouses);
  
  unbuffer_all_connections();

  if (errcode == 0)
    freelog(LOG_NORMAL, "houses sold in monop %d by %s", monid, plr->name());
  
}

void Game::handle_jail_free_request(Player *plr, 
				    struct packet_generic_integer *pack)
{
  int event = pack->value;


  freelog(LOG_NORMAL, "jail free request from %s, eventid:%d", 
	  plr->name(), event);

  
  // wrong syntax
  if (! (event==E_LEAVE_JAIL_PAY_FINE || event==E_LEAVE_JAIL_USE_CARD) ) {
    notify_player(plr, "Game: wrong event in jail free request (internal)");
    freelog(LOG_NORMAL, "error: no game yet");
    return;
  }

  // valid request ?
  if (_curr_plr != plr->id()) {
    notify_player(plr, "Game: It's not your turn, wait for %s",
		  _player[_curr_plr]->name());
    freelog(LOG_NORMAL, "error: not his turn");
    return;
  } 

  if (!plr->is_in_jail()) {	// plr isn't in jail ?
    notify_player(plr, "Game: You are not in jail");
    freelog(LOG_NORMAL, "error: is not in jail");
    return;
  }
  
  // further tests are performed in Player::leave_jail
  // (e.g. if plr has enough money or if plr has any jail free card)

  switch (event) {

  case E_LEAVE_JAIL_PAY_FINE:
    if (plr->leave_jail(E_LEAVE_JAIL_PAY_FINE))
      return;	// on error -> return here

    freelog(LOG_NORMAL, "%s pays fine and leaves jail ", plr->name());
    break;

  case E_LEAVE_JAIL_USE_CARD:
    if (plr->leave_jail(E_LEAVE_JAIL_USE_CARD))
      return;	// on error -> return here

    freelog(LOG_NORMAL, "%s uses card and leaves jail ", plr->name());
    break;
  }

}

void Game::handle_pay_request(int plr_no)
{
  assert(!nobody(plr_no));

  Player *plr = _player[plr_no];

  freelog(LOG_VERBOSE, "pay request packet from %s", plr->name());
  
  if (plr_no != _curr_plr) {
    notify_player(plr, "Game: It's not your turn, wait for %s",
		  _player[_curr_plr]->name());
    freelog(LOG_NORMAL, "error: not his turn");
    return;
  } else if (plr->debts_sum() == 0) { // player has no debts
    notify_player(plr, "Game: You have no debts.");
    freelog(LOG_NORMAL, "error: has no debts");
    return;
  }
  else if (plr->settle_debts() != 0) { // most likely, player isn't afloat
    notify_player(plr,("Game: You can't settle your debts.\n"
		       "Game: Try selling houses, mortgaging properties etc.\n"
			"Game: or declare yourself bankrupt") );
    freelog(LOG_NORMAL, "error: settling debts not possible: "
	    "most likely not afloat");
    return;
  } 

  // OK
  freelog(LOG_NORMAL, "%s has settled his debts", plr->name());

}

void Game::handle_declare_bankrupt(Player *plr)
{
  int i;

  Player *dest = 0;
  
  freelog(LOG_VERBOSE, "declare bankrupt packet from %s", plr->name());
  
  if (plr != _player[_curr_plr]) {
    notify_player(plr, "Game: It's not your turn, wait for %s.",
 		  _player[_curr_plr]->name());
    freelog(LOG_NORMAL, "error: not his turn");
    return;
  } else if (!plr->is_bankrupt()) {
    notify_player(plr, "Game: You are not bankrupt. Play to the very end." );
    freelog(LOG_NORMAL, "error: not bankrupt");
    return;
  } 
  
  // OK

  freelog(LOG_VERBOSE, "%s goes bankrupt", plr->name());

  // find out the creditor
  
  
  if (plr->bank_debts() > 0) { // we owe to the bank
    dest = BANK;
  } else {  // we owe to another player
    for (i=0; i<nplayers(); ++i) {
      if (plr->debts(i)) {
	dest = _player[i];
      }
    }
    assert(dest); // we should have debts to one player!
    // TODO: we should not have debts to more than one player!!
  }

  // transactions of cash, holdings etc. to creditor
  plr->go_bankrupt(dest);

  // clear all trades of that player
  for (i=0; i<_nplayers; ++i) {
    clear_trade(plr, _player[i]);
  }

  plr->die();

  freelog(LOG_NORMAL, "%s has declared himself bankrupt", plr->name());
  freelog(LOG_NORMAL, "%s is now dead", plr->name());

  // is there anything left to do? No.
  // we trigger the end of plr's turn automatically
  // see main.cc: turn loop
  force_end_of_sniff=1;

}


void Game::handle_trade_request(Player *plr, struct packet_trade *pack)
{
  assert(plr);

  int tradeid = pack->tradeid;
  int event = pack->event;

  freelog(LOG_NORMAL, "trade request from %s: tradeid=%d, event=%d", 
	  plr->name(), tradeid, event);

  int errcode;
  errcode = check_trade_packet_consistency(plr, pack);
  if (errcode != 0) {
    freelog(LOG_NORMAL, "error: trade packet check failed: errcode=%d",
	    errcode);
    return;
  }

  int money = pack->money;

  //  plr trades with whom?
  Player *destplr = _player[pack->destid];

  Trade *trade = get_trade(plr, destplr);

  buffer_all_connections();

  switch (event) {

  case E_TRADE_OFFER:
    assert(trade == 0); // we have already tested consistency of packet

    trade = new Trade(plr, destplr, money, pack->prop);
    set_trade(plr, destplr, trade);
    break;

  case E_TRADE_REPLY:
    assert(trade);  // we have already tested consistency of packet
    trade->reply(plr, money, pack->prop);
    break;

  case E_TRADE_AGREE:
    assert(trade);  // we have already tested consistency of packet
    trade->agree(plr);
    if (trade->is_finished())
      clear_trade(plr, destplr);	// Trade was executed, clear it
    break;

  case E_TRADE_REJECT:
    assert(trade);  // we have already tested consistency of packet
    trade->reject(plr);
    if (trade->is_finished())
      clear_trade(plr, destplr);
    break;

  default:

    assert(0);		// bug
  }

  unbuffer_all_connections();

}

void Game::handle_anim_finished(Player *plr)
{
  int i;

  freelog(LOG_VERBOSE, "anim finished packet from %s", plr->name());

  if (!anim_in_progress()) {
    notify_player(plr, "Game: There is no anim in progress");
    freelog(LOG_NORMAL, "error: no anim in progress");
    return;
  }
  
  // OK
  int plr_no;
  for (int i=0; i<_nplayers; ++i) if (plr == _player[i]) plr_no=i;
  
  _plr_anim_finished[plr_no] = true;
  
  freelog(LOG_NORMAL, "%s has finished anim", plr->name());

  // TODO: timeout for client answers

}



/*
 * game flow
 */

void Game::next_turn()
{
  Player *currplr = _player[_curr_plr]; // initalized with old currplr

  if (_ndoubles == 0) {	// current player didn't roll doubles, so next plr
    // cycle
    _curr_plr++;
    _curr_plr %= _nplayers;
    currplr = _player[_curr_plr];
  } 

  while (!currplr->is_alive()) {  // leave out all dead players
    _curr_plr++;
    _curr_plr %=_nplayers;
    currplr = _player[_curr_plr];
  }

  // increase nturns_in_jail for curr_plr
  if (currplr->location() == JAIL)
    currplr->inc_nturns_in_jail();

  _rolled_dice = false;

  buffer_all_connections();

  send_player_info(TOALL, currplr, E_YOUR_TURN);
  send_your_turn(TOALL, _curr_plr);

  unbuffer_all_connections();
}

void Game::pre_dice()
{
  Player *currplr = _player[_curr_plr];
  if (currplr->is_ai_controlled()) {
    currplr->ai_wait(1);
    currplr->ai_roll_dice();
  } else {  // wait until current player has rolled dice
    force_end_of_sniff=0;
    freelog(LOG_DEBUG, "sniffingpackets in pre_dice()");
    while(sniff_packets()==1);
  }

}

void Game::post_dice()
{
  Player *currplr = _player[_curr_plr];
  if (currplr->is_ai_controlled()) {
    currplr->ai_wait(1);
    currplr->ai_settle_debts();
    currplr->ai_leave_jail();
    currplr->ai_manage_holdings();
    currplr->ai_end_turn();
    currplr->ai_wait(1);
  } else {  // wait until current player has rolled dice
    force_end_of_sniff=0;
    freelog(LOG_DEBUG, "sniffingpackets in post_dice()");
    while(sniff_packets()==1);
  }
}


void Game::end_turn()
{
  freelog(LOG_VERBOSE, "end of turn reached.");
  // TODO: evtl. Auktion etc.
}


void Game::roll_dice(int dicevalue[], const int dice_sum)
{
  int *diceval;
  static bool always_roll_doubles = false;	// for debugging jail

  if (dicevalue == 0)	// roll dice in turn (e.g. no card)
    diceval = _last_dice;
  else
    diceval = dicevalue;

//    diceval[0] = myrand(6) + 1;
//    diceval[1] = myrand(6) + 1;
  
  
  if (always_roll_doubles) diceval[0] = diceval[1] = 2;
  else {
    diceval[0] = 1 + (int) (6.0 * rand()/(RAND_MAX + 1.0));
    diceval[1] = 1 + (int) (6.0 * rand()/(RAND_MAX + 1.0));
  }

  if (CHEATING_ALLOWED && (dice_sum >= 2 && dice_sum <= 12) ) {
    (dice_sum >= 7) ? (diceval[0] = 6) : diceval[0] = dice_sum-1;
    diceval[1] = dice_sum - diceval[0];
    freelog(LOG_NORMAL, "plr %s is cheating!", _player[_curr_plr]->name());
    notify_player(TOALL, "Game: %s is cheating!", _player[_curr_plr]->name());
  }
  
  if (dicevalue == 0) {
    _rolled_dice = true;
    if (diceval[0] == diceval[1])
      _ndoubles++;
    else
      _ndoubles = 0;
  }
  
//    freelog(LOG_NORMAL, "dice rolled"); // logged already in handle_dice...

  send_dice_rolled(TOALL, diceval);
}


/*
 * misc
 */


void Game::buffer_all_connections()
{
  int i;

  for(i=0;i<_nplayers;i++)
    connection_do_buffer(_player[i]->conn());
}

void Game::unbuffer_all_connections()
{
  int i;

  for(i=0;i<_nplayers;i++)
    connection_do_unbuffer(_player[i]->conn());
}



/*
 * ...
 */
void Game::send_your_turn(Player *dest, int plr_id)
{
  int o;
  struct packet_generic_integer pack;
  
  pack.value = plr_id;

  freelog(LOG_VERBOSE, "send_your_turn: %d", plr_id);
  for(o=0; o<_nplayers; o++)
    if(!dest || _player[o]==dest) {
      send_packet_generic_integer(_player[o]->conn(), 
				  PACKET_YOUR_TURN, &pack);
    }
}

void Game::send_dice_rolled(Player *dest, int dicevalue[])
{
  int o;
  struct packet_dice_rolled pack;

  pack.playerid = _curr_plr;
  pack.dice1 = dicevalue[0];
  pack.dice2 = dicevalue[1];

  freelog(LOG_VERBOSE, "send_dice_rolled: %d, dice1=%d, dice2=%d", 
	    _curr_plr, dicevalue[0], dicevalue[1]);

  for(o=0; o<_nplayers; o++)
    if(!dest || _player[o]==dest)
      send_packet_dice_rolled(_player[o]->conn(), &pack);
}


void Game::send_card_drawn(Player *dest, Player *src, 
			   int cardtype, int card_no)
{
  int o;
  struct packet_generic_values pack;

  pack.id = src->id();
  pack.value1 = cardtype;	// COMCHEST or CHANCE card ?
  pack.value2 = card_no;

  freelog(LOG_VERBOSE, "send_card_drawn: who=%d, type=%d, card_no=%d", 
	    src->id(), cardtype, card_no);

  for(o=0; o<_nplayers; o++)
    if(!dest || _player[o]==dest)
      send_packet_generic_values(_player[o]->conn(), PACKET_CARD_DRAWN, &pack);

}


void Game::send_trade(Player *dest, Trade *trade, int event, int offer_from)
{
  int i;
  struct packet_trade pack;
  

  freelog(LOG_VERBOSE, "send_trade: tradeid=%d, event=%d, offer_from=%d", 
	  trade->id(), event, offer_from);

  for(i=0; i<_nplayers; i++)		/* dests */  
    if(!dest || _player[i]==dest) {
      trade->prepare_packet_for(_player[i], &pack, event, offer_from);
      send_packet_trade(_player[i]->conn(), &pack);
    }
}




/**************************************************************************
dest can be 0 meaning all players
**************************************************************************/
void Game::send_game_init_info(Player *dest)
{
  int o;
  struct packet_game_init_info ginfo;
  
  freelog(LOG_VERBOSE, "send_game_init_info");

  for(o=0; o<_nplayers; o++)           
    if(!dest || _player[o]==dest) {
      init_info_for(_player[o], &ginfo);
      send_packet_game_init_info(_player[o]->conn(), &ginfo);
    }
}


/**************************************************************************
dest can be 0 meaning all players
**************************************************************************/
void Game::send_game_info(Player *dest)
{
  int o;
  struct packet_game_info ginfo;
  
  freelog(LOG_VERBOSE, "send_game_info");

  for(o=0; o<_nplayers; o++)           
    if(!dest || player(o)==dest) {
      info_for(_player[o], &ginfo);
      send_packet_game_info(_player[o]->conn(), &ginfo);
    }
}

/**************************************************************************
dest and/or obj can be 0 meaning all players
**************************************************************************/
void Game::send_player_init_info(Player *dest, Player *obj)
{
  assert(0);	// not needed, because game_init_info includes this

  int o, i;
  struct packet_player_init_info pinfo;
  
  for(o=0; o<_nplayers; o++)           // dests
    if(!dest || _player[o]==dest)
      for (i=0; i<_nplayers; i++) {	// objs
	if (!obj || _player[i]==obj) {
	  _player[i]->init_info_for(_player[o], &pinfo);
	  send_packet_player_init_info(_player[o]->conn(), &pinfo);
	}
      }
}

/**************************************************************************
dest and/or obj can be 0 meaning all players
**************************************************************************/
void Game::send_player_info(Player *dest, Player *obj, int event)
{
  int o, i;
  struct packet_player_info pinfo;
  
  for(o=0; o<_nplayers; o++)		/* dests */  
    if(!dest || _player[o]==dest)
      for (i=0; i<_nplayers; i++) {	/* objs */
	if (!obj || _player[i]==obj) {
	  _player[i]->info_for(_player[o], &pinfo, event);
	  send_packet_player_info(_player[o]->conn(), &pinfo);
	}
      }
}


/**************************************************************************
dest can be 0 meaning all players
**************************************************************************/
void Game::send_field_info(Player *dest, Field *field, int event)
{
  int o;
  struct packet_field_info finfo;
  
  for(o=0; o<_nplayers; o++)           
    if(!dest || _player[o]==dest) {
      field->info_for(_player[o], &finfo, event);
      send_packet_field_info(_player[o]->conn(), &finfo);
    }
}


/**************************************************************************
dest can be 0 meaning all players
**************************************************************************/
void Game::send_monop_info(Player *dest, Monop *monop, int event)
{
  int o;
  struct packet_monop_info minfo;
  
  for(o=0; o<_nplayers; o++)           
    if(!dest || _player[o]==dest) {
      monop->info_for(_player[o], &minfo, event);
      send_packet_monop_info(_player[o]->conn(), &minfo);
    }
}



/**************************************************************************
dest can be 0 meaning all players
**************************************************************************/
void Game::send_all_info(Player *dest)
{
  int i, j;

  /* infos, that describe the server state entirely */
  send_game_init_info(dest);
  send_game_info(dest);

  // trade infos
  for (i=0; i<_nplayers; ++i)
    for (j=0; j<_nplayers; ++j)
      if (_trade[i][j]) {
	send_trade(dest, _trade[i][j], E_NOEVENT, T_SRC);
	send_trade(dest, _trade[i][j], E_NOEVENT, T_DEST);
      }
	
  if (_curr_plr != -1) {
    send_your_turn(dest, _curr_plr);
    if (_rolled_dice) send_dice_rolled(dest, _last_dice);
  }
  
}


// ptype is packet type (begin, end, confirm_anim)
void Game::send_anim(Player *dest, int ptype, int anim_id)
{
  int o;
  struct packet_generic_integer pack;
  
  assert(ptype == PACKET_CONFIRM_ANIM_FINISHED);

  pack.value = anim_id;

  freelog(LOG_VERBOSE, "send_anim: %d", ptype);
  for(o=0; o<_nplayers; o++)
    if(!dest || _player[o]==dest) {
      send_packet_generic_integer(_player[o]->conn(), 
				  ptype, &pack);
    }
}

void Game::wait_for_anim_finished()
{
  int i;
  freelog(LOG_VERBOSE, "wait for anim finished");
  for (i=0; i<_nplayers; ++i) {
    if (_player[i]->is_connected()) _plr_anim_finished[i] = false;
    else _plr_anim_finished[i] = true;  // disconnected players don't animate!
  }
}


int Game::complement_event(const int event) const
{
  switch (event) {
  case E_PAY_RENT: return E_RECV_RENT; break;
  case E_GO_BANKRUPT: return E_BANKRUPT_CREDITOR; break;
  default: return event;
  }

}

