// -*- C++ -*-
/***************************************************************************
                          monop.h  -  description
                             -------------------
    begin                : Fre Jun 22 17:19:03 CEST 2001
    copyright            : (C) 2001 by Daniel Hermann
    email                : Daniel.Hermann@stud.uni-karlsruhe.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef MONOP_H
#define MONOP_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdlib.h>

#include "global.h"

#include "street.h"

class Player;

class Monop {
  
public:

  /*
   * field : Field[BOARD_N_FIELDS], street_idx : Street[MAX_STREETS_IN_MONOP]
   */
  Monop(int id, Field *[], const int street_idx[]);

  const int id() const
    { return _id; }
  const int nstreets() const
    { return _nstreets; }

  Street * const street(const int street_no) const
    { 
      assert(street_no >= 0 && street_no < MAX_STREETS_IN_MONOP);
      return _street[street_no]; 
    }

  bool no_houses()
    { 
      int i;
      for (i=0; i<_nstreets; ++i) 
	if (_street[i]->nhouses() > 0) return false;
      return true;
    }
  bool has_houses() { return !no_houses(); }

  const bool is_monop() const
    { return (_owner != NULL); }
  const bool is_yours(Player *plr) const
    { return (_owner == plr); }

  bool check();

  int buy_houses(Player *, int nhouses[]);
  int sell_houses(Player *, int nhouses[]);

  int sell_one_house(Player *);
  int sell_all_houses(Player *);

  void init_info_for(Player *player, struct packet_monop_init_info *);
  void info_for(Player *player, struct packet_monop_info *, int event);

protected:
  
  int _id;
  int _nstreets; // number of streets belonging to monopoly
  Street *_street[MAX_STREETS_IN_MONOP];

  Player *_owner;

};





#endif /* MONOP_H */
