// -*- C++ -*-
/***************************************************************************
                          property.h  -  description
                             -------------------
    begin                : Fre Jun 22 17:19:03 CEST 2001
    copyright            : (C) 2001 by Daniel Hermann
    email                : Daniel.Hermann@stud.uni-karlsruhe.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef PROPERTY_H
#define PROPERTY_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "global.h"
#include "field.h"

class Property : public Field {
  
public:

  Property(const int location, const int price);

  Property(struct packet_field_init_info *);

  virtual ~Property();

  Player * const owner() const 
    { return _owner; }
  const int price() const 
    { return _price; }
  const int unmortgage_price() const
    { return _price/2 + (int) (PROPERTY_UNMORTGAGE_FACTOR * _price); }

  const bool is_mortgaged() const 
    { return _is_mortgaged; }
  const bool is_owned() const
    { return (_owner) ? true : false; }
  const bool is_yours(Player *p) const
    { return (p == _owner) ? true : false; }

  virtual const int rent() const = 0;

  virtual void enter(Player *);

  virtual void pay_rent(Player *);

  virtual int buy(Player *);

  virtual int mortgage(Player *);
  virtual int unmortgage(Player *);
  
  virtual int change_owner(Player *new_owner);

  virtual int const status(const Player *) const;
  virtual int const actions(const Player *) const;

  // init_info_for is defined in derived classes of Property

  virtual void info_for(const Player *, struct packet_field_info *, int event);

protected:
  
  Player *_owner;
  int _price;
  bool _is_mortgaged;


};

#endif /* PROPERTY_H */
