// -*- C++ -*-
/***************************************************************************
                          street.h  -  description
                             -------------------
    begin                : Fre Jun 22 17:19:03 CEST 2001
    copyright            : (C) 2001 by Daniel Hermann
    email                : Daniel.Hermann@stud.uni-karlsruhe.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef STREET_H
#define STREET_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <assert.h>

#include "global.h"
#include "property.h"

class Monop;
class Player;

class Street : public Property {

public:

  Street(const int location, 
	 const int price, const int house_price,
	 const int rent[]);

  Street(struct packet_field_init_info *);

  Monop * const monop() const 
    { return _monop; }
  const int nhouses() const 
    { return _nhouses; }
  const int house_price() const
    { return _house_price; }

  const int rent() const 
    { return ( (_is_monop && _nhouses==0) ? 2 * _rent[0] 
	       : _rent[_nhouses] ); }

  const int rent_for(int nh) const
    { 
      assert(nh>=0 && nh<STREET_N_RENT_CLASSES);
      return _rent[nh]; 
    }

  const  void set_monop(Monop *monop) 
    { _monop = monop; }

  // only use on server side!!
  void set_is_monop(bool flag)
    { _is_monop = flag; }





  virtual int buy(Player *);

  virtual int mortgage(Player *);

  virtual int change_owner(Player *new_owner);

  virtual int buy_houses(Player *, const int nhouses);
  virtual int sell_houses(Player *, const int nhouses);

  virtual int const status(const Player *) const;
  virtual int const actions(const Player *) const;

  virtual void init_info_for(const Player *, struct packet_field_init_info *);
  virtual void info_for(const Player *, struct packet_field_info *, int event);

protected:

  Monop *_monop;
  int _rent[STREET_N_RENT_CLASSES]; 
  int _house_price;

  int _nhouses;
  bool _is_monop;
  
};

#endif /* STREET_H */
