// -*- C++ -*-
/***************************************************************************
                          trade.h  -  description
                             -------------------
    begin                : Fre Jun 22 17:19:03 CEST 2001
    copyright            : (C) 2001 by Daniel Hermann
    email                : Daniel.Hermann@stud.uni-karlsruhe.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef TRADE_H
#define TRADE_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>

#include "player.h"

enum {
  T_NOTRADE=0, // must be an integer which is never possible for a statusflag
  T_SRC,
  T_DEST
};

enum {
  T_NOT_FINISHED = 0,
  T_SRC_REJECTED,
  T_DEST_REJECTED,
  T_SRC_AGREED,
  T_DEST_AGREED,
  T_PROCEEDED
};

class Trade {
  
public:
  
  Trade(Player *src, Player *dest, int money_src, int prop_src[]);

  Trade(struct packet_trade *);

  Trade(const Trade&);	// not defined!

  int reply(Player *plr, int money, int prop[]);
  
  void agree(Player *);
  void reject(Player *);

  Player * const get_src() const
    { return srcplr; }
  Player * const get_dest() const
    { return destplr; }

  const bool is_finished() const
    { return (_finishedflag != T_NOT_FINISHED); }

  bool offer_is_valid(Player *fromplr);
  void test_money_in_offer(Player *plr);
  /*
   * packets in/out
   */
  void prepare_packet_for(Player *, struct packet_trade *, int event,
			  int take_offer);

  void update(struct packet_trade *);

  const int id() const { return _id; }

protected:
  
  const int _id;
  Player * const srcplr;
  Player * const destplr;
  
  int _srcmoney;
  int _destmoney;
  int _srcprop[BOARD_N_FIELDS];
  int _destprop[BOARD_N_FIELDS];

  int _last_offer;	// only used on server side (yet)
  bool _dest_replied_once;	// to ensure that there was at least one reply

  int _finishedflag;

  static int count;

protected:
  
  const char *srcplrname() { return srcplr->name(); }
  const char *destplrname() { return destplr->name(); }

  bool const src_gives(int *prop, int i) const;
  bool const dest_gives(int *prop, int i) const;

  int finish_trade();


};


#endif /* TRADE_H */
