// -*- C++ -*-
/***************************************************************************
                          main.cpp  -  description
                             -------------------
    begin                : Fre Jun 22 17:19:03 CEST 2001
    copyright            : (C) 2001 by Daniel Hermann
    email                : Daniel.Hermann@stud.uni-karlsruhe.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

//
// System
//

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <assert.h>
#include <time.h>

#include <unistd.h>
#include <sys/types.h>

// don't know if this is needed !
#ifdef __EMX__
#include <netdb.h>
#include <sys/ioctl.h>
#include <sys/termio.h>
#include <termios.h>
#endif


//
// Common + Handler
//

#include "global.h"
#include "shared.h"
#include "version.h"
#include "events.h"
#include "log.h"
#include "packets.h"

#include "console.h"
#include "sernet.h"
#include "stdinhand.h"
#include "plrhand.h"
#include "chathand.h"
#include "settings.h"

//
// Game
//

#include "player.h"
#include "game.h"

#include "gamelog.h"

#include "main.h"

extern struct connection connections[];

enum server_states server_state = BOOT_STATE;

Game *game;

/* this global is checked deep down the netcode. 
   packets handling functions can set it to none-zero, to
   force end-of-tick asap
*/
int force_end_of_sniff;

int is_server = 1;

int port=DEFAULT_SOCK_PORT;

bool first_player_gets_cmdlevel_hack = false;

/**************************************************************************
...
**************************************************************************/
int main(int argc, char *argv[])
{
  int h=0, v=0;
  char *log_filename=NULL;
  char *gamelog_filename=NULL;
  int i;
  int loglevel=LOG_NORMAL;

  if (!getuid() || !geteuid()) {
    fprintf(stderr, "%s: Fatal error: you're trying to run me as superuser!\n",
	    (argv[0] ? argv[0] : "Capitalist server"));
    fprintf(stderr,"Use a non-privileged account instead.\n");
    exit(1);
  }

  /* no  we don't use GNU's getopt or even the "standard" getopt */
  /* yes we do have reasons ;)                                   */
  
  i=1;
  while(i<argc) {
    if(!strcmp("-h", argv[i]) || !strcmp("--help", argv[i])) { 
      h=1;
      break;
    }
    else if(!strcmp("-l", argv[i]) || !strcmp("--log", argv[i])) { 
      if(++i<argc) 
	log_filename=argv[i];
      else {
	fprintf(stderr, "Error: filename not specified.\n");
	h=1;
	break;
      }
    }
    else if(!strcmp("-g", argv[i]) || !strcmp("--gamelog", argv[i])) {
      if(++i<argc)
        gamelog_filename=argv[i];
      else {
        fprintf(stderr, "Error: Game log filename not specified.\n");
        h=1;
        break;
      }
    }
    else if(!strcmp("-p", argv[i]) || !strcmp("--port", argv[i])) { 
      if(++i<argc) 
	port=atoi(argv[i]);
      else {
	fprintf(stderr, "Error: port not specified.\n");
	h=1;
	break;
      }
    }
    else if(!strcmp("-d", argv[i]) || !strcmp("--debug", argv[i])) { 
      if(++i<argc) {
	loglevel = log_parse_level_str(argv[i]);
	if (loglevel == -1) {
	  loglevel = LOG_NORMAL;
	  h=1;
	  break;
	}
      } else {
	fprintf(stderr, "Error: no debug log level specified.\n");
	h=1;
	break;
      }
    }
    else if(!strcmp("-v", argv[i]) || !strcmp("--version", argv[i])) { 
      v=1;
    }
    else if(!strcmp("--firsthack", argv[i])) { 
      first_player_gets_cmdlevel_hack = true;
      break;
    }
    else {
      fprintf(stderr, "Error: unknown option '%s'\n", argv[i]);
      h=1;
      break;
    }
    i++;
  }
    
  if(v && !h) {
    fprintf(stderr, CAPITALIST_NAME_VERSION "\n");
    exit(0);
  }

  con_dump(C_COMMENT, "\n");
  con_write(C_VERSION, "This is the server for %s", CAPITALIST_NAME_VERSION);
  con_write(C_COMMENT, "Get more info at %s",
	    WEBSITE_URL);
  con_dump(C_COMMENT, "\n");
  if(h) {
/*      fprintf(stderr, "  -g, --gamelog F\tUse F as game logfile\n"); */
    fprintf(stderr, "  -h, --help\t\tPrint a summary of the options\n");
    fprintf(stderr, "  -l, --log F\t\tUse F as logfile\n");
    fprintf(stderr, "  -p, --port N\t\tconnect to port N\n");
#ifdef DEBUG
    fprintf(stderr, "  -d, --debug N\t\tSet debug log level (0,1,2,3,"
	                                "or 3:file1,min,max:...)\n");
#else
    fprintf(stderr, "  -d, --debug N\t\tSet debug log level (0,1,2)\n");
#endif
    fprintf(stderr, "  -v, --version\t\tPrint the version number\n");
    fprintf(stderr, "Report bugs to <%s>.\n", BUG_EMAIL_ADDRESS);
    exit(0);
  }

  con_log_init(log_filename, loglevel);
  gamelog_init(gamelog_filename);
  gamelog_set_level(GAMELOG_FULL);
  gamelog(GAMELOG_NORMAL,"Starting new log");
  
  con_flush();

  /* init network */  
  init_connections(); 
  server_open_socket();

  set_server_state(PRE_GAME_STATE);

  /* accept new players */
  freelog(LOG_NORMAL, "Now accepting new client connections");
  if (first_player_gets_cmdlevel_hack)
    freelog(LOG_NORMAL, "First player gets command access level HACK");

  /* wait for serverop to start..*/
  while(server_state==PRE_GAME_STATE)
    sniff_packets();
  
  /* here: all players have joined game, game was started by operator */
  
  /* TODO: implement SELECT_TOKEN_STATE */

  /* generate AI players */
  generate_ai_players();

  /* run the game */

  set_server_state(RUN_GAME_STATE);

  /* game running loop */
  while(server_state==RUN_GAME_STATE) {

    game->next_turn();

    game->pre_dice();

    /* here: dice rolled accepted */
    
    game->post_dice();

    /* here: turn done accepted */

    freelog(LOG_DEBUG, "Endturn");
    game->end_turn();


    int nplayers_alive = 0;
    for (i=0; i<game->nplayers(); ++i) {
      if (game->player(i)->is_alive()) {
	nplayers_alive++;
      }
    }

    if (nplayers_alive < 2) {
      set_server_state(GAME_OVER_STATE);
    }

  }

  while(server_state==GAME_OVER_STATE) {
    force_end_of_sniff=0;
    sniff_packets();
  }

  // impossible case here !
  // TODO: allow new game

  close_connections_and_socket();
  exit(2);
  
  return 0;
}

/**************************************************************************
...
**************************************************************************/
void set_server_state(enum server_states newstate)
{
  if(server_state!=newstate) {
    server_state=newstate;

    switch (newstate) {

    case PRE_GAME_STATE:
      delete game;	// tabula rasa
      game = new Game();  // no Players, no Board yet
      send_game_state(TOALL, CLIENT_PRE_GAME_STATE);
      break;
    case SELECT_TOKEN_STATE:
      send_game_state(TOALL, CLIENT_SELECT_TOKEN_STATE);
      break;
    case RUN_GAME_STATE:
      assert(game);	// there should already be a game

      send_game_state(TOALL, CLIENT_GAME_RUNNING_STATE);
      //initialize Board, Players get unique ID's, initialize Cards
      //send clients all information
      game->run();	

      break;
      
    case GAME_OVER_STATE:
      notify_player(TOALL, "Game: The game is over ... "
		    "It was nice to play with you :-)");
      send_game_state(TOALL, CLIENT_GAME_OVER_STATE);
      break;
    default:
      assert(0);	// bug: no other states possible

    }
    force_end_of_sniff = 1;	// server should notice the change
  }

}


/**************************************************************************
...
**************************************************************************/

void start_game(void)
{
}


/**************************************************************************
...
**************************************************************************/
void handle_packet_input(struct connection *pconn, char *packet, int type)
{
  int i;
  Player *pplayer=NULL;

  switch(type) {
  case PACKET_REQUEST_JOIN_GAME:	/* handled here */
    handle_request_join_game(pconn, (struct packet_req_join_game *)packet);
    free(packet);
    return;
    break;
  }

  for(i=0; i<game->nplayers(); i++)
    if(game->player(i)->conn()==pconn) {
      pplayer=game->player(i);
      break;
    }

  if(i==game->nplayers()) {
    freelog(LOG_VERBOSE, "got game packet from unaccepted connection");
    free(packet);
    return;
  }

  if ( ( server_state != RUN_GAME_STATE ) &&
       ( (type == PACKET_TURN_DONE) ||
	 (type == PACKET_DICE_ROLL_REQUEST) ||
	 (type == PACKET_PROPERTY_BUY) ||
	 (type == PACKET_PROPERTY_MORTGAGE) ||
	 (type == PACKET_PROPERTY_UNMORTGAGE) ||
	 (type == PACKET_HOUSES_BUY) ||
	 (type == PACKET_HOUSES_SELL) ||
	 (type == PACKET_JAIL_FREE_REQUEST) ||
	 (type == PACKET_PAY_REQUEST) ||
	 (type == PACKET_DECLARE_BANKRUPT) ||
	 (type == PACKET_TRADE) ) )
    {
    notify_player(pplayer, "Game hasn't started yet");
    freelog(LOG_NORMAL, "error: no game yet");
    free(packet);
    return;
  }


  // for dead players, allow chat and animation confirmations but nothing more
  if(!pplayer->is_alive() && 
     (type!=PACKET_CHAT_MSG) &&
     (type!=PACKET_ANIM_FINISHED)
     ) {
    notify_player(pplayer, "Game: You are dead! Be quiet.");
    freelog(LOG_VERBOSE, "%s sends still packets, rejected", pplayer->name());
    free(packet);
    return;
  }
  
  // reject some request types if anim in progress
  if ( game->anim_in_progress() && 
       ( (type == PACKET_TURN_DONE) ||
	 (type == PACKET_DICE_ROLL_REQUEST) ||
	 (type == PACKET_PROPERTY_BUY) ||
	 (type == PACKET_PROPERTY_MORTGAGE) ||
	 (type == PACKET_PROPERTY_UNMORTGAGE) ||
	 (type == PACKET_HOUSES_BUY) ||
	 (type == PACKET_HOUSES_SELL) ||
	 (type == PACKET_JAIL_FREE_REQUEST) ||
	 (type == PACKET_PAY_REQUEST) ||
	 (type == PACKET_DECLARE_BANKRUPT) ) )
    {
      notify_player(pplayer, 
		    "Game: Anim in progress. Wait for others to finish.");
      freelog(LOG_NORMAL, "error: anim in progress");
      free(packet);
      return;
    }

  switch(type) {
    
  case PACKET_TURN_DONE:	/* handled in game.cpp */
    game->handle_turn_done(pplayer);
    break;

  case PACKET_DICE_ROLL_REQUEST:	/* handled in game.cpp */
    game->handle_dice_roll_request(pplayer, 
				   (struct packet_generic_integer *)packet);
    break;

  case PACKET_PROPERTY_BUY:
    game->handle_property_buy(pplayer, 
			      (struct packet_generic_integer *)packet);
    break;
 
  case PACKET_PROPERTY_MORTGAGE:
    game->handle_property_mortgage(pplayer, 
				   (struct packet_generic_integer *)packet);
    break;

  case PACKET_PROPERTY_UNMORTGAGE:
    game->handle_property_unmortgage(pplayer, 
				     (struct packet_generic_integer *)packet);
    break;

//    case PACKET_PROPERTY_INFO:
//      handle_property_info(pplayer, (struct packet_property_info *)packet);
//      break;

  case PACKET_HOUSES_BUY:
    game->handle_houses_buy(pplayer, 
			    (struct packet_house_request *)packet);
    break;
    
  case PACKET_HOUSES_SELL:
    game->handle_houses_sell(pplayer, 
			     (struct packet_house_request *)packet);
    break;

  case PACKET_JAIL_FREE_REQUEST:
    game->handle_jail_free_request(pplayer, 
				   (struct packet_generic_integer *)packet);
    break;

  case PACKET_PAY_REQUEST:	/* handled in game.cpp */
    game->handle_pay_request(i);
    break;
    
  case PACKET_DECLARE_BANKRUPT:	/* handled in game.cpp */
    game->handle_declare_bankrupt(pplayer);
    break;
    
//    case PACKET_HOLDINGS_INFO:
//      handle_holdings_info(pplayer, (struct packet_holdings_info *)packet);
//      break;

//    case PACKET_BUDGET_INFO:
//      handle_budget_info(pplayer, (struct packet_budget_info *)packet);
//      break;

//    case PACKET_JAIL_CARD:
//      handle_jail_card(pplayer, (struct packet_jail_request *)packet);
//      break;

//    case PACKET_JAIL_PAY:
//      handle_jail_pay(pplayer, (struct packet_jail_request *)packet);
//      break;

  case PACKET_TRADE:
    game->handle_trade_request(pplayer, (struct packet_trade *)packet);
    break;
    
//    case PACKET_REPORT_REQUEST:
//      handle_report_request(pplayer, 
//       ((struct packet_generic_integer *)packet)->value);
//      break;

  case PACKET_CHAT_MSG:		/* handled in chathand.cpp */
    handle_chat_msg(pplayer, (struct packet_generic_message *)packet);
    break;

  case PACKET_ANIM_FINISHED:	/* handled in game.cpp */
    game->handle_anim_finished(pplayer);
    break;

  default:
    freelog(LOG_NORMAL, "uh got an unknown packet from %s", game->player(i)->name());
  }

  free(packet);
}

/**************************************************************************
 Send a join_game reply, accepting the player.
 (Send to pplayer->conn(), which should be set, as should pplayer->addr())
 'rejoin' is whether this is a new player, or re-connection.
 Prints a log message, and notifies other players.
**************************************************************************/
void join_game_accept(Player *pplayer, int rejoin)
{
  struct packet_join_game_reply packet;
  int i;

  static bool first_player = true;
  
  assert(pplayer);
  assert(pplayer->is_connected());

  packet.you_can_join = 1;
  sprintf(packet.message, "%s %s.", (rejoin?"Welcome back":"Welcome"),
				     pplayer->name());
  send_packet_join_game_reply(pplayer->conn(), &packet);
  freelog(LOG_NORMAL, "<%s@%s> has %sjoined the game.",
	  pplayer->name(), pplayer->addr(), (rejoin?"re":""));
  for(i=0; i<game->nplayers(); ++i) {
    if (pplayer != game->player(i)) {
      notify_player(game->player(i),
	  	    "Game: Player <%s@%s> has %sconnected.",
		    pplayer->name(), pplayer->addr(), (rejoin?"re":""));
    }
  }

  // if (!first_player)  // \todo: FIXME
  //  pplayer->set_ai_controlled(true);

  // with option --firsthack the first player accepted gets command level
  // "hack"

  if (first_player_gets_cmdlevel_hack && first_player) {
    pplayer->conn()->access_level = ALLOW_HACK;
    notify_player(pplayer, "Game: you now have access level '%s'",
		  cmdlevel_name(ALLOW_HACK));
  }

  // finally unset first_player
  first_player = false;

}

/**************************************************************************
 Introduce player to server, and notify of other players.
**************************************************************************/
void introduce_game_to_player(Player *pplayer)
{
  char hostname[512];
  Player *cplayer;
  int i, nconn, nother;
  
  if (!pplayer->is_connected())
    return;
  
  if (gethostname(hostname, 512)==0) {
    notify_player(pplayer, "Welcome to the %s Server\nrunning at %s", 
		  CAPITALIST_NAME_VERSION, hostname);
  } else {
    notify_player(pplayer, "Welcome to the %s Server",
		  CAPITALIST_NAME_VERSION);
  }
  if (server_state==RUN_GAME_STATE) {
    /* if the game is running, players can just view the Players menu */
    return;
  }
  for(nconn=0, i=0; i<game->nplayers(); ++i) {
    nconn += (int) (game->player(i)->is_connected());
  }
  if (nconn != 1) {
    notify_player(pplayer, "There are currently %d players connected:", nconn);
  } else {
    notify_player(pplayer, "There is currently 1 player connected:");
  }
  for(i=0; i<game->nplayers(); ++i) {
    cplayer = game->player(i);
    if (cplayer->is_connected()) {
      notify_player(pplayer, "  <%s@%s>%s", cplayer->name(), cplayer->addr(),
		    ((!cplayer->is_alive()) ? " (R.I.P.)" : ""));
    }
  }
  nother = game->nplayers() - nconn;
  if (nother > 0) {
    if (nother == 1) {
      notify_player(pplayer, "There is 1 other player:");
    } else {
      notify_player(pplayer, "There are %d other players:", nother);
    }
    for(i=0; i<game->nplayers(); ++i) {
      cplayer = game->player(i);
      if (!cplayer->is_connected()) {
	notify_player(pplayer, "  %s%s", cplayer->name(), 
		      ((!cplayer->is_alive()) ? " (R.I.P.)" : ""));
      }
    }
  }
  /* notify_player(pplayer, "Waiting for the server to start the game."); */
  /* I would put this here, but then would need another message when
     the game is started. --dwp */
}

/**************************************************************************
  This is used when a new player joins a game, before the game
  has started.  If pconn is NULL, is an AI, else a client. (no AI's yet!!)
  ...
**************************************************************************/
void accept_new_player(char *name, struct connection *pconn)
{
  Player *pplayer = new Player(name, game->startbudget());

  pplayer->connect(pconn);

  if (pconn) {
    join_game_accept(pplayer, 0);
  } else {	// AI-player
    // nothing needs to be done here
  }

  game->add_player(pplayer);

  introduce_game_to_player(pplayer);

  // if game is full, start it automatically
  if (server_state==PRE_GAME_STATE && game->nplayers() == GAME_MAX_PLAYERS) {
    freelog(LOG_NORMAL, "game full, started automatically");
    set_server_state(SELECT_TOKEN_STATE);
  }

}
  
/**************************************************************************
...
**************************************************************************/

void reject_new_player(char *msg, struct connection *pconn)
{
  struct packet_join_game_reply packet;
  
  packet.you_can_join=0;
  strcpy(packet.message, msg);
  send_packet_join_game_reply(pconn, &packet);
}
  
/**************************************************************************
...
**************************************************************************/

void handle_request_join_game(struct connection *pconn, 
			      struct packet_req_join_game *req)
{
  Player *pplayer;
  char msg[MSG_SIZE];
  
  freelog(LOG_NORMAL, 
	  "Connection request from %s with client version %d.%d.%d",
	  req->name, req->major_version, req->minor_version, 
	  req->patch_version);
  
  connection_do_buffer(pconn);

  if((pplayer=game->find_player_by_name(req->name))) {

    if(!pplayer->is_connected()) {
      pplayer->connect(pconn);

      join_game_accept(pplayer, 1);
      introduce_game_to_player(pplayer);

      if(server_state==RUN_GAME_STATE) {
        send_game_state(pplayer, CLIENT_GAME_RUNNING_STATE);
	game->send_all_info(pplayer);
      }
      connection_do_unbuffer(pconn);
      return;
    }

    if(server_state==PRE_GAME_STATE) {
      if(game->nplayers()==GAME_MAX_PLAYERS) {	// is game full?
	reject_new_player("Sorry you can't join. The game is full.", pconn);
	freelog(LOG_NORMAL, "game full - %s was rejected.", req->name);    
	connection_do_unbuffer(pconn);
	close_connection(pconn);
        return;
      }
      else {	// you have same name as another connected player -> reject
	/* Used to have here: accept_new_player(req->name, pconn); 
	 * but duplicate names cause problems even in PRE_GAME_STATE
	 * because they are used to identify players for various
	 * server commands. --dwp
	 */
	reject_new_player("Sorry, someone else already has that name.",
			  pconn);
	freelog(LOG_NORMAL, "%s was rejected, name already used.", req->name);
	connection_do_unbuffer(pconn);
	close_connection(pconn);

        return;
      }
    }

    // game is running and you want to connect with a valid name
    sprintf(msg, "You can't join the game. %s is already connected.", 
	    pplayer->name());
    reject_new_player(msg, pconn);
    freelog(LOG_NORMAL, "%s (second connection) was rejected.", 
	    pplayer->name());
    connection_do_unbuffer(pconn);
    close_connection(pconn);

    return;
  }

  /* here: unknown name -> previously unknown player */

  // you can't join running game
  if(server_state!=PRE_GAME_STATE) {
    reject_new_player("Sorry you can't join. The game is already running.",
		      pconn);
    freelog(LOG_NORMAL, "game running - %s was rejected.", req->name);
    connection_do_unbuffer(pconn);
    close_connection(pconn);

    return;
  }

  // you can't join full game (should be a very rare case, because game 
  // starts automatically if it's full
  // note: only possible SELECT_TOKEN_STATE ?

  if(game->nplayers()==GAME_MAX_PLAYERS) {
    reject_new_player("Sorry you can't join. The game is full.", pconn);
    freelog(LOG_NORMAL, "game full - %s was rejected.", req->name);    
    connection_do_unbuffer(pconn);
    close_connection(pconn);

    return;
  }


  // last not least: accept all other cases

  accept_new_player(req->name, pconn);    
  connection_do_unbuffer(pconn);
}

/**************************************************************************
...
**************************************************************************/

void lost_connection_to_player(struct connection *pconn)
{
  int i;

  for(i=0; i<game->nplayers(); i++)
    if(game->player(i)->conn()==pconn) {
      game->player(i)->disconnect();
      freelog(LOG_NORMAL, "lost connection to %s", game->player(i)->name());
      game->send_player_info(TOALL, game->player(i), E_LOST_CONNECTION);
      notify_player(TOALL, "Game: Lost connection to %s", game->player(i)->name());

      if(server_state==PRE_GAME_STATE || server_state==SELECT_TOKEN_STATE)
	server_remove_player(game->player(i));
      else {  /* RUN_GAME_STATE */
      /* TODO: convert player to AI ?? */
      }
      game->set_anim_finished(i, true);
      return;
    }

  freelog(LOG_FATAL, "lost connection to unknown");
}


void server_remove_player(Player *pplayer)
{
  struct packet_generic_integer pack;
  int o, idx;
  
  freelog(LOG_NORMAL, "plr %s has been removed from the game", 
	  pplayer->name());

  notify_player(pplayer, "Game: You've been removed from the game!");
  if(pplayer->conn())
    close_connection(pplayer->conn());
  pplayer->disconnect();
  notify_player(TOALL, "Game: %s has been removed from the game", 
		pplayer->name());
  
  for (o=0; o<game->nplayers(); o++) {
    if (game->player(o)==pplayer) 
      idx=o;
  }
  pack.value=idx;

  for(o=0; o<game->nplayers(); o++)
    send_packet_generic_integer(game->player(o)->conn(), PACKET_REMOVE_PLAYER,
				&pack);

  game->remove_player(idx);

  /* TODO: is there something left to clean up ?? */
}

/**************************************************************************
generate_ai_players() - add newly generated AI players to the game
   We check to see if the server option "aifill" is greater
   than the number of players currently connected.  If so, we create the
   appropriate number of players (server_settings::aifill - game->nplayers())
   from scratch, choosing an appropriate name for each.
**************************************************************************/
void generate_ai_players(void)
{
  char player_name[PLAYER_MAX_LENGTH_NAME];
  Player *pplayer;
  int i, old_nplayers;

  if (server_settings::aifill > GAME_MAX_PLAYERS) {
    server_settings::aifill = GAME_MAX_PLAYERS;
    freelog(LOG_NORMAL,
	    "Maxplayers smaller than aifill; aifill reduced to %d.",
            GAME_MAX_PLAYERS);
  }
  int name_id=0;
  for(;game->nplayers() < server_settings::aifill;) {

    old_nplayers = game->nplayers();
    do {
      snprintf(player_name, PLAYER_MAX_LENGTH_NAME, "Computer%d", ++name_id);
    } while(game->find_player_by_name(player_name));
    accept_new_player(player_name, 0);
    freelog(LOG_NORMAL, "%s has been added as an AI-controlled player.",
            player_name);
    notify_player(NULL, "Game: %s has been added as an AI-controlled player.",
		  player_name);

    pplayer=game->find_player_by_name(player_name);
    if (!((game->nplayers() == old_nplayers+1) && pplayer
	  && strcmp(player_name, pplayer->name())==0)) {
      con_write(C_FAIL, "Error creating new AI player: %s\n",
		player_name);
      break;			/* don't loop forever */
    }

    pplayer->set_ai_controlled(true);

  }
}

/**************************************************************************
...
**************************************************************************/
void send_game_state(Player *dest, int state)
{
  int o;
  struct packet_generic_integer pack;

  pack.value=state;
  
  
  for(o=0; o<game->nplayers(); o++)
    if(!dest || game->player(o)==dest)
      send_packet_generic_integer(game->player(o)->conn(), 
				  PACKET_GAME_STATE, &pack);
}



