// -*- C++ -*-

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "global.h"
#include "shared.h"
#include "settings.h"

/**
 * Values for the static variables of server_settings
 */
int server_settings::aifill = GAME_DEFAULT_AIFILL;


#define GEN_BOOL(name, value, sclass, func, default)		\
  {name, sclass, SSET_BOOL,			\
      &value, default, func,			\
      NULL, 0, NULL, 0, 0,			\
      NULL, NULL, NULL, 0},

#define GEN_INT(name, value, sclass, func, min, max, default)	\
  {name, sclass, SSET_INT,			\
      NULL, false, NULL,			\
      &value, default, func, min, max,		\
      NULL, NULL, NULL, 0},

#define GEN_STRING(name, value, sclass, func, default)		\
  {name, sclass, SSET_STRING,			\
      NULL, false, NULL,			\
      NULL, 0, NULL, 0, 0,			\
      value, default, func, sizeof(value)},

#define GEN_END							\
  {NULL, SSET_LAST, SSET_INT,			\
      NULL, false, NULL,			\
      NULL, 0, NULL, 0, 0,			\
      NULL, NULL, NULL},


struct settings_s settings[] = {

  /* These should be grouped by sclass */

  /* Options affecting numbers of players and AI players.  These only
   * affect the start of the game and can not be adjusted after that.
   */
  GEN_INT("aifill", server_settings::aifill, SSET_PLAYERS, NULL,
	  GAME_MIN_PLAYERS, GAME_MAX_PLAYERS, GAME_DEFAULT_AIFILL)

  /* Game initialization parameters (only affect the first start of the game,
   * and not reloads).  Can not be changed after first start of game.
   */
//   GEN_INT("startbudget", game->opt_startbudget, SSET_GAME_INIT, NULL,
// 	  GAME_MIN_STARTBUDGET, GAME_MAX_STARTBUDGET, GAME_DEFAULT_STARTBUDGET)

  /* TODO: here we can put rules
   */

  /* Other server settings (SSET_META) */
//   GEN_BOOL("autotoggle", server_settings.auto_ai_toggle, SSET_META,
// 	   autotoggle_callback, GAME_DEFAULT_AUTO_AI_TOGGLE)

  GEN_END
};

/* The number of settings, not including the END. */
const int SETTINGS_NUM = ARRAY_SIZE(settings) - 1;
