// -*- C++ -*-

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SETTINGS_H
#define SETTINGS_H

#include <stddef.h>

/* Server setting types. */
enum sset_type {
  SSET_BOOL, SSET_INT, SSET_STRING
};

/* The following classes determine what can be changed when.
 * Actually, some of them have the same "changeability", but
 * different types are separated here in case they have
 * other uses.
 * See the settings[] array for what these correspond to and
 * explanations. (TODO: implement help strings)
 */
enum sset_class {
  SSET_PLAYERS,
  SSET_GAME_INIT,
  SSET_RULES,
  SSET_META,
  SSET_LAST
};

struct settings_s {
  char *name;
  enum sset_class sclass;
  enum sset_type type;
    /*** bool part ***/
  bool *bool_value;
  bool bool_default_value;
  bool (*bool_validate)(bool value, const char **reject_message);

  /*** int part ***/
  int *int_value;
  int int_default_value;
  bool (*int_validate)(int value, const char **reject_message);
  int int_min_value, int_max_value;

  /*** string part ***/
  char *string_value;
  const char *string_default_value;
  bool (*string_validate)(const char * value, const char **reject_message);
  size_t string_value_size;	/* max size we can write into string_value */
};

extern struct settings_s settings[];
extern const int SETTINGS_NUM;

/**
 * This is the actual structure containing only the values of the server options.
 * The values for the static variables are defined in settings.cpp. 
 */
struct server_settings {
  static int aifill;
//   int auto_ai_toggle = GAME_DEFAULT_AUTO_AI_TOGGLE;
};


#endif  /* SETTINGS_H */
